############################ Copyrights and license ############################
#                                                                              #
# Copyright 2018 Benoit Latinier <benoit@latinier.fr>                          #
# Copyright 2018 Yossarian King <yggy@blackbirdinteractive.com>                #
# Copyright 2019 Benoit Latinier <benoit@latinier.fr>                          #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Jody McIntyre <scjody@modernduck.com>                         #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework


class Project(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user().get_repo("PyGithub")
        self.proj = self.g.get_project(1682941)

    # See https://developer.github.com/v3/projects/#get-a-project
    def testAttributes(self):
        self.assertEqual(self.proj.body, "To be used for testing project access API for PyGithub.")
        self.assertEqual(self.proj.columns_url, "https://api.github.com/projects/1682941/columns")
        self.assertEqual(self.proj.created_at, datetime(2018, 8, 1, 4, 6, 57, tzinfo=timezone.utc))
        self.assertEqual(self.proj.creator.login, "bbi-yggy")
        self.assertEqual(self.proj.html_url, "https://github.com/bbi-yggy/PyGithub/projects/1")
        self.assertEqual(self.proj.id, 1682941)
        self.assertEqual(self.proj.name, "TestProject")
        self.assertEqual(repr(self.proj), 'Project(name="TestProject")')
        self.assertEqual(self.proj.node_id, "MDc6UHJvamVjdDE2ODI5NDE=")
        self.assertEqual(self.proj.number, 1)
        self.assertIsNone(self.proj.organization_permission)
        self.assertEqual(self.proj.owner_url, "https://api.github.com/repos/bbi-yggy/PyGithub")
        self.assertIsNone(self.proj.private)
        self.assertEqual(self.proj.state, "open")
        self.assertEqual(self.proj.updated_at, datetime(2018, 8, 3, 0, 31, 17, tzinfo=timezone.utc))
        self.assertEqual(self.proj.url, "https://api.github.com/projects/1682941")

    def testLazyAttributes(self):
        project = self.g.withLazy(True).get_project(42)
        self.assertEqual(str(project), "Project(name=None)")
        self.assertEqual(project.id, 42)
        self.assertEqual(project.url, "/projects/42")

    def testGetOrganizationProjects(self):
        expectedProjects = ["Project1", "Project2", "Project3"]
        org = self.g.get_organization("PyGithubTestOrg")
        projects = [proj.name for proj in org.get_projects("open")]
        self.assertEqual(projects, expectedProjects)

    def testGetRepositoryProjects(self):
        expectedProjects = ["TestProject", "TestProjectClosed"]
        projects = [proj.name for proj in self.repo.get_projects("all")]
        self.assertEqual(projects, expectedProjects)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2026 Matt Davis <35502728+matt-davis27@users.noreply.github.com>   #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github.OrganizationSecretScanAlert
import github.PaginatedList
import github.Repository

from . import Framework


class OrganizationSecretScanAlert(Framework.TestCase):
    alert: github.OrganizationSecretScanAlert.OrganizationSecretScanAlert

    def setUp(self):
        super().setUp()
        self.org = self.g.get_organization(
            "github"
        )  # This is a placeholder organization that should always exist, switch this to your own test organization

    def testMultipleAlerts(self):
        multiple_alerts = self.org.get_secret_scanning_alerts()
        self.assertIsInstance(multiple_alerts, github.PaginatedList.PaginatedList)
        self.assertIsInstance(multiple_alerts[0], github.OrganizationSecretScanAlert.OrganizationSecretScanAlert)
        alert_list = [alert for alert in multiple_alerts]
        self.assertEqual(len(alert_list), 2)  # Update this when more alerts are added

        test_alert = alert_list[-1]
        self.assertEqual(test_alert.number, 1)
        self.assertEqual(test_alert.created_at, datetime(2020, 11, 6, 18, 18, 30, tzinfo=timezone.utc))
        self.assertEqual(test_alert.url, "https://api.github.com/repos/owner/repo/secret-scanning/alerts/1")
        self.assertEqual(test_alert.html_url, "https://github.com/owner/repo/security/secret-scanning/1")
        self.assertEqual(
            test_alert.locations_url,
            "https://api.github.com/repos/owner/private-repo/secret-scanning/alerts/1/locations",
        )
        self.assertEqual(test_alert.state, "open")
        self.assertIsNone(test_alert.resolution)
        self.assertIsNone(test_alert.resolved_at)
        self.assertIsNone(test_alert.resolved_by)
        self.assertEqual(test_alert.secret_type, "mailchimp_api_key")
        self.assertEqual(test_alert.secret, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-us2")
        self.assertIsNone(test_alert.push_protection_bypassed_by)
        self.assertFalse(test_alert.push_protection_bypassed)
        self.assertIsNone(test_alert.push_protection_bypassed_at)
        self.assertIsNone(test_alert.push_protection_bypass_request_reviewer)
        self.assertIsNone(test_alert.push_protection_bypass_request_reviewer_comment)
        self.assertIsNone(test_alert.push_protection_bypass_request_comment)
        self.assertIsNone(test_alert.push_protection_bypass_request_html_url)
        self.assertIsNone(test_alert.resolution_comment)
        self.assertEqual(test_alert.validity, "unknown")
        self.assertFalse(test_alert.publicly_leaked)
        self.assertFalse(test_alert.multi_repo)
        self.assertFalse(test_alert.is_base64_encoded)
        self.assertEqual(test_alert.first_location_detected.path, "/example/secrets.txt")
        self.assertEqual(test_alert.first_location_detected.start_line, 1)
        self.assertEqual(test_alert.first_location_detected.end_line, 1)
        self.assertEqual(test_alert.first_location_detected.start_column, 1)
        self.assertEqual(test_alert.first_location_detected.end_column, 64)
        self.assertEqual(test_alert.first_location_detected.blob_sha, "af5626b4a114abcb82d63db7c8082c3c4756e51b")
        self.assertEqual(
            test_alert.first_location_detected.blob_url,
            "https://api.github.com/repos/octocat/hello-world/git/blobs/af5626b4a114abcb82d63db7c8082c3c4756e51b",
        )
        self.assertEqual(test_alert.first_location_detected.commit_sha, "f14d7debf9775f957cf4f1e8176da0786431f72b")
        self.assertEqual(
            test_alert.first_location_detected.commit_url,
            "https://api.github.com/repos/octocat/hello-world/git/commits/f14d7debf9775f957cf4f1e8176da0786431f72b",
        )
        self.assertFalse(test_alert.has_more_locations)
        self.assertIsInstance(test_alert.repository, github.Repository.Repository)
        self.assertEqual(test_alert.repository.name, "Hello-World")
        self.assertEqual(test_alert.repository.full_name, "octocat/Hello-World")

    def testRepr(self):
        multiple_alerts = self.org.get_secret_scanning_alerts()
        alert_list = [alert for alert in multiple_alerts]
        test_alert = alert_list[-1]
        self.assertEqual(repr(test_alert), "OrganizationSecretScanAlert(number=1)")

    def testGetAlertsWithAllArguments(self):
        multiple_alerts = self.org.get_secret_scanning_alerts(
            state="resolved",
            secret_type="adafruit_io_key",
            resolution="false_positive",
            sort="created",
            direction="asc",
            validity="inactive",
            is_publicly_leaked=False,
            is_multi_repo=False,
            hide_secret=True,
        )
        alert_list = [alert for alert in multiple_alerts]
        self.assertEqual(len(alert_list), 1)

        test_alert = alert_list[-1]
        self.assertIsNone(test_alert.secret)  # Because hide_secret is True



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github

from . import Framework


class ProjectCard(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user().get_repo("PyGithub")
        self.proj = self.g.get_project(1682941)
        self.cols = self.proj.get_columns()
        self.col = self.cols[1]
        cards = self.col.get_cards()
        self.pull_card = cards[0]
        self.issue_card = cards[1]
        self.note_card = cards[2]
        self.edit_col = self.g.get_project(4015343).get_columns()[0]

    # See https://developer.github.com/v3/projects/cards/#get-a-project-card
    def testAttributes(self):
        card = self.pull_card
        self.assertFalse(card.archived)
        self.assertIsNone(card.column_name)
        self.assertEqual(card.column_url, "https://api.github.com/projects/columns/3138831")
        self.assertEqual(card.content_url, "https://api.github.com/repos/bbi-yggy/PyGithub/issues/1")
        self.assertEqual(card.created_at, datetime(2018, 8, 1, 4, 53, 59, tzinfo=timezone.utc))
        self.assertEqual(card.creator, self.repo.owner)
        self.assertEqual(card.created_at.year, 2018)
        self.assertEqual(card.id, 11780055)
        self.assertEqual(card.node_id, "MDExOlByb2plY3RDYXJkMTE3ODAwNTU=")
        self.assertIsNone(card.note, None)
        self.assertIsNone(card.project_id)
        self.assertIsNone(card.project_url)
        self.assertEqual(card.updated_at, datetime(2018, 8, 1, 4, 54, 16, tzinfo=timezone.utc))
        self.assertEqual(card.url, "https://api.github.com/projects/columns/cards/11780055")
        self.assertEqual(repr(card), "ProjectCard(id=11780055)")

    def testGetContent(self):
        pull = self.pull_card.get_content("PullRequest")
        self.assertIsInstance(pull, github.PullRequest.PullRequest)
        self.assertEqual(pull.title, "Work in progress on support for GitHub projects API.")
        self.assertRaises(ValueError, self.pull_card.get_content, "foo")

        issue = self.issue_card.get_content()
        self.assertIsInstance(issue, github.Issue.Issue)
        self.assertEqual(issue.title, "Test issue")

        note_content = self.note_card.get_content()
        self.assertEqual(note_content, None)

    def testMove(self):
        cols = self.cols
        card = self.pull_card
        self.assertTrue(card.move("top", cols[2].id))
        self.assertTrue(card.move("bottom", cols[1]))

    def testDelete(self):
        card = self.pull_card
        self.assertTrue(card.delete())

    def testGetAll(self):
        expectedProjects = ["TestProject"]
        expectedCards = 5
        projects = []
        cards = 0
        for proj in self.repo.get_projects():
            projects.append(proj.name)
            for col in proj.get_columns():
                for card in col.get_cards("all"):
                    cards += 1
        self.assertEqual(projects, expectedProjects)
        self.assertEqual(cards, expectedCards)

    def testCreateWithNote(self):
        project = self.repo.create_project("Project created by PyGithub", "Project Body")
        column = project.create_column(
            "Project Column created by PyGithub",
        )
        card1 = column.create_card(note="Project Card")
        self.assertEqual(card1.id, 16039019)

    def testCreateFromIssue(self):
        project = self.repo.create_project("Project created by PyGithub", "Project Body")
        column = project.create_column(
            "Project Column created by PyGithub",
        )
        issue = self.repo.create_issue(title="Issue created by PyGithub")
        card2 = column.create_card(content_id=issue.id, content_type="Issue")
        self.assertEqual(card2.id, 16039106)

    def testEditWithoutParameters(self):
        card = self.edit_col.create_card(note="Project Card")
        card.edit()

    def testEditNote(self):
        card = self.edit_col.create_card(note="Project Card")
        card.edit(note="Edited Card")
        self.assertEqual(card.note, "Edited Card")

    def testEditArchived(self):
        card = self.edit_col.create_card(note="Project Card")
        card.edit(archived=True)
        self.assertEqual(card.archived, True)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Balázs Rostás <rostas.balazs@gmail.com>                       #
# Copyright 2017 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2018 Alice GIRARD <bouhahah@gmail.com>                             #
# Copyright 2018 Bruce Richardson <itsbruce@workshy.org>                       #
# Copyright 2018 Jacopo Notarstefano <jacopo.notarstefano@gmail.com>           #
# Copyright 2018 Riccardo Pittau <elfosardo@users.noreply.github.com>          #
# Copyright 2018 Shubham Singh <41840111+singh811@users.noreply.github.com>    #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Rigas Papathanasopoulos <rigaspapas@gmail.com>                #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 Surya Teja <94suryateja@gmail.com>                            #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Anuj Bansal <bansalanuj1996@gmail.com>                        #
# Copyright 2020 Glenn McDonald <testworksau@users.noreply.github.com>         #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2021 MeggyCal <MeggyCal@users.noreply.github.com>                  #
# Copyright 2021 秋葉 <ambiguous404@gmail.com>                                   #
# Copyright 2022 KimSia Sim <245021+simkimsia@users.noreply.github.com>        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Chris Wells <ping@cwlls.com>                                  #
# Copyright 2024 Eduardo Ramírez <edu.rh90@gmail.com>                          #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Oskar Jansson <56458534+janssonoskar@users.noreply.github.com>#
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

import github

from . import Framework


class AuthenticatedUser(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.user = self.g.get_user()

    def testAttributes(self):
        self.assertEqual(
            self.user.avatar_url,
            "https://secure.gravatar.com/avatar/b68de5ae38616c296fa345d2b9df2225?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-140.png",
        )
        self.assertEqual(self.user.bio, "")
        self.assertEqual(self.user.blog, "http://vincent-jacques.net")
        self.assertIsNone(self.user.business_plus)
        self.assertEqual(self.user.collaborators, 0)
        self.assertEqual(self.user.company, "Criteo")
        self.assertEqual(
            self.user.created_at,
            datetime(2010, 7, 9, 6, 10, 6, tzinfo=timezone.utc),
        )
        self.assertEqual(self.user.disk_usage, 16692)
        self.assertEqual(self.user.email, "vincent@vincent-jacques.net")
        self.assertIsNone(self.user.events_url)
        self.assertEqual(self.user.followers, 13)
        self.assertIsNone(self.user.followers_url)
        self.assertEqual(self.user.following, 24)
        self.assertIsNone(self.user.following_url)
        self.assertIsNone(self.user.gists_url)
        self.assertEqual(self.user.gravatar_id, "b68de5ae38616c296fa345d2b9df2225")
        self.assertFalse(self.user.hireable)
        self.assertEqual(self.user.html_url, "https://github.com/jacquev6")
        self.assertEqual(self.user.id, 327146)
        self.assertIsNone(self.user.ldap_dn)
        self.assertEqual(self.user.location, "Paris, France")
        self.assertEqual(self.user.login, "jacquev6")
        self.assertEqual(self.user.name, "Vincent Jacques")
        self.assertIsNone(self.user.notification_email)
        self.assertIsNone(self.user.organizations_url)
        self.assertEqual(self.user.owned_private_repos, 5)
        self.assertEqual(self.user.plan.name, "micro")
        self.assertEqual(self.user.plan.collaborators, 1)
        self.assertEqual(self.user.plan.space, 614400)
        self.assertEqual(self.user.plan.private_repos, 5)
        self.assertEqual(self.user.private_gists, 5)
        self.assertEqual(self.user.public_gists, 1)
        self.assertEqual(self.user.public_repos, 10)
        self.assertIsNone(self.user.received_events_url)
        self.assertIsNone(self.user.repos_url)
        self.assertIsNone(self.user.site_admin)
        self.assertIsNone(self.user.starred_url)
        self.assertIsNone(self.user.subscriptions_url)
        self.assertEqual(self.user.total_private_repos, 5)
        self.assertIsNone(self.user.twitter_username)
        self.assertEqual(self.user.type, "User")
        self.assertIsNone(self.user.updated_at)
        self.assertEqual(self.user.url, "https://api.github.com/users/jacquev6")
        self.assertEqual(self.user.node_id, "MDQ6VXNlcjMyNzE0Ng==")
        self.assertEqual(repr(self.user), 'AuthenticatedUser(login="jacquev6")')
        self.assertTrue(self.user.two_factor_authentication)
        self.assertIsNone(self.user.user_view_type)

    def testEditWithoutArguments(self):
        self.user.edit()

    def testEditWithAllArguments(self):
        self.user.edit(
            "Name edited by PyGithub",
            "Email edited by PyGithub",
            "Blog edited by PyGithub",
            "Company edited by PyGithub",
            "Location edited by PyGithub",
            True,
            "Bio edited by PyGithub",
        )
        self.assertEqual(self.user.name, "Name edited by PyGithub")
        self.assertEqual(self.user.email, "Email edited by PyGithub")
        self.assertEqual(self.user.blog, "Blog edited by PyGithub")
        self.assertEqual(self.user.company, "Company edited by PyGithub")
        self.assertEqual(self.user.location, "Location edited by PyGithub")
        self.assertTrue(self.user.hireable)
        self.assertEqual(self.user.bio, "Bio edited by PyGithub")

    def testEmails(self):
        emails = self.user.get_emails()
        self.assertEqual(
            [item.email for item in emails],
            ["vincent@vincent-jacques.net", "github.com@vincent-jacques.net"],
        )
        self.assertTrue(emails[0].primary)
        self.assertTrue(emails[0].verified)
        self.assertEqual(emails[0].visibility, "private")
        self.user.add_to_emails("1@foobar.com", "2@foobar.com")
        self.assertEqual(
            [item.email for item in self.user.get_emails()],
            [
                "vincent@vincent-jacques.net",
                "1@foobar.com",
                "2@foobar.com",
                "github.com@vincent-jacques.net",
            ],
        )
        self.user.remove_from_emails("1@foobar.com", "2@foobar.com")
        self.assertEqual(
            [item.email for item in self.user.get_emails()],
            ["vincent@vincent-jacques.net", "github.com@vincent-jacques.net"],
        )

    def testFollowing(self):
        nvie = self.g.get_user("nvie")
        self.assertListKeyEqual(
            self.user.get_following(),
            lambda u: u.login,
            [
                "schacon",
                "jamis",
                "chad",
                "unclebob",
                "dabrahams",
                "jnorthrup",
                "brugidou",
                "regisb",
                "walidk",
                "tanzilli",
                "fjardon",
                "r3c",
                "sdanzan",
                "vineus",
                "cjuniet",
                "gturri",
                "ant9000",
                "asquini",
                "claudyus",
                "jardon-u",
                "s-bernard",
                "kamaradclimber",
                "Lyloa",
                "nvie",
            ],
        )
        self.assertTrue(self.user.has_in_following(nvie))
        self.user.remove_from_following(nvie)
        self.assertFalse(self.user.has_in_following(nvie))
        self.user.add_to_following(nvie)
        self.assertTrue(self.user.has_in_following(nvie))
        self.assertListKeyEqual(
            self.user.get_followers(),
            lambda u: u.login,
            [
                "jnorthrup",
                "brugidou",
                "regisb",
                "walidk",
                "afzalkhan",
                "sdanzan",
                "vineus",
                "gturri",
                "fjardon",
                "cjuniet",
                "jardon-u",
                "kamaradclimber",
                "L42y",
            ],
        )

    def testWatching(self):
        gitflow = self.g.get_user("nvie").get_repo("gitflow")
        self.assertListKeyEqual(
            self.user.get_watched(),
            lambda r: r.name,
            [
                "git",
                "boost.php",
                "capistrano",
                "boost.perl",
                "git-subtree",
                "git-hg",
                "homebrew",
                "celtic_knot",
                "twisted-intro",
                "markup",
                "hub",
                "gitflow",
                "murder",
                "boto",
                "agit",
                "d3",
                "pygit2",
                "git-pulls",
                "django_mathlatex",
                "scrumblr",
                "developer.github.com",
                "python-github3",
                "PlantUML",
                "bootstrap",
                "drawnby",
                "django-socketio",
                "django-realtime",
                "playground",
                "BozoCrack",
                "FatherBeaver",
                "PyGithub",
                "django",
                "django",
                "TestPyGithub",
            ],
        )
        self.assertTrue(self.user.has_in_watched(gitflow))
        self.user.remove_from_watched(gitflow)
        self.assertFalse(self.user.has_in_watched(gitflow))
        self.user.add_to_watched(gitflow)
        self.assertTrue(self.user.has_in_watched(gitflow))

    def testStarring(self):
        gitflow = self.g.get_user("nvie").get_repo("gitflow")
        self.assertListKeyEqual(
            self.user.get_starred(),
            lambda r: r.name,
            [
                "git",
                "boost.php",
                "capistrano",
                "boost.perl",
                "git-subtree",
                "git-hg",
                "homebrew",
                "celtic_knot",
                "twisted-intro",
                "markup",
                "hub",
                "gitflow",
                "murder",
                "boto",
                "agit",
                "d3",
                "pygit2",
                "git-pulls",
                "django_mathlatex",
                "scrumblr",
                "developer.github.com",
                "python-github3",
                "PlantUML",
                "bootstrap",
                "drawnby",
                "django-socketio",
                "django-realtime",
                "playground",
                "BozoCrack",
                "FatherBeaver",
                "amaunet",
                "django",
                "django",
                "moviePlanning",
                "folly",
            ],
        )
        self.assertTrue(self.user.has_in_starred(gitflow))
        self.user.remove_from_starred(gitflow)
        self.assertFalse(self.user.has_in_starred(gitflow))
        self.user.add_to_starred(gitflow)
        self.assertTrue(self.user.has_in_starred(gitflow))

    def testSubscriptions(self):
        gitflow = self.g.get_user("nvie").get_repo("gitflow")
        self.assertListKeyEqual(
            self.user.get_subscriptions(),
            lambda r: r.name,
            [
                "gitflow",
                "ViDE",
                "Boost.HierarchicalEnum",
                "QuadProgMm",
                "DrawSyntax",
                "DrawTurksHead",
                "PrivateStuff",
                "vincent-jacques.net",
                "Hacking",
                "C4Planner",
                "developer.github.com",
                "PyGithub",
                "PyGithub",
                "django",
                "CinePlanning",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "IpMap",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
                "PyGithub",
            ],
        )
        self.assertTrue(self.user.has_in_subscriptions(gitflow))
        self.user.remove_from_subscriptions(gitflow)
        self.assertFalse(self.user.has_in_subscriptions(gitflow))
        self.user.add_to_subscriptions(gitflow)
        self.assertTrue(self.user.has_in_subscriptions(gitflow))

    def testGetAuthorizations(self):
        self.assertListKeyEqual(self.user.get_authorizations(), lambda a: a.id, [372294])

    def testCreateRepository(self):
        repo = self.user.create_repo(name="TestPyGithub")
        self.assertEqual(repo.url, "https://api.github.com/repos/jacquev6/TestPyGithub")

    def testCreateProject(self):
        project = self.user.create_project(name="TestPyGithub", body="This is the body")
        self.assertEqual(project.url, "https://api.github.com/projects/4084610")

    def testCreateRepositoryWithAllArguments(self):
        repo = self.user.create_repo(
            name="TestPyGithub",
            description="Repo created by PyGithub",
            homepage="http://foobar.com",
            private=False,
            has_issues=False,
            has_projects=False,
            has_wiki=False,
            has_discussions=False,
            has_downloads=False,
            allow_squash_merge=False,
            allow_merge_commit=False,
            allow_rebase_merge=True,
            delete_branch_on_merge=False,
        )
        self.assertEqual(repo.url, "https://api.github.com/repos/jacquev6/TestPyGithub")

    def testCreateRepositoryWithAutoInit(self):
        repo = self.user.create_repo(name="TestPyGithub", auto_init=True, gitignore_template="Python")
        self.assertEqual(repo.url, "https://api.github.com/repos/jacquev6/TestPyGithub")

    def testCreateAuthorizationWithoutArguments(self):
        authorization = self.user.create_authorization()
        self.assertEqual(authorization.id, 372259)

    def testCreateAuthorizationWithAllArguments(self):
        authorization = self.user.create_authorization(
            ["repo"], "Note created by PyGithub", "http://vincent-jacques.net/PyGithub"
        )
        self.assertEqual(authorization.id, 372294)

    def testCreateAuthorizationWithClientIdAndSecret(self):
        # I don't have a client_id and client_secret so the ReplayData for this test is forged
        authorization = self.user.create_authorization(
            client_id="01234567890123456789",
            client_secret="0123456789012345678901234567890123456789",
        )
        self.assertEqual(authorization.id, 372294)

    def testCreateGist(self):
        gist = self.user.create_gist(
            True,
            {"foobar.txt": github.InputFileContent("File created by PyGithub")},
            "Gist created by PyGithub",
        )
        self.assertEqual(gist.description, "Gist created by PyGithub")
        self.assertEqual(list(gist.files.keys()), ["foobar.txt"])
        self.assertEqual(gist.files["foobar.txt"].content, "File created by PyGithub")

    def testCreateGistWithoutDescription(self):
        gist = self.user.create_gist(True, {"foobar.txt": github.InputFileContent("File created by PyGithub")})
        self.assertEqual(gist.description, None)
        self.assertEqual(list(gist.files.keys()), ["foobar.txt"])
        self.assertEqual(gist.files["foobar.txt"].content, "File created by PyGithub")

    def testCreateKey(self):
        key = self.user.create_key(
            "Key added through PyGithub",
            "ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAQEA2Mm0RjTNAYFfSCtUpO54usdseroUSIYg5KX4JoseTpqyiB/hqewjYLAdUq/tNIQzrkoEJWSyZrQt0ma7/YCyMYuNGd3DU6q6ZAyBeY3E9RyCiKjO3aTL2VKQGFvBVVmGdxGVSCITRphAcsKc/PF35/fg9XP9S0anMXcEFtdfMHz41SSw+XtE+Vc+6cX9FuI5qUfLGbkv8L1v3g4uw9VXlzq4GfTA+1S7D6mcoGHopAIXFlVr+2RfDKdSURMcB22z41fljO1MW4+zUS/4FyUTpL991es5fcwKXYoiE+x06VJeJJ1Krwx+DZj45uweV6cHXt2JwJEI9fWB6WyBlDejWw== vincent@IDEE",
        )
        self.assertEqual(key.id, 2626650)

    def testGetEvents(self):
        self.assertListKeyBegin(
            self.user.get_events(),
            lambda e: e.type,
            ["PushEvent", "IssuesEvent", "IssueCommentEvent", "PushEvent"],
        )

    def testGetOrganizationEvents(self):
        self.assertListKeyBegin(
            self.user.get_organization_events(self.g.get_organization("BeaverSoftware")),
            lambda e: e.type,
            ["CreateEvent", "CreateEvent", "PushEvent", "PushEvent"],
        )

    def testGetGists(self):
        self.assertListKeyEqual(
            self.user.get_gists(),
            lambda g: g.id,
            [
                "2793505",
                "2793179",
                "11cb445f8197e17d303d",
                "1942384",
                "dcb7de17e8a52b74541d",
            ],
        )
        self.assertListKeyEqual(
            self.user.get_gists(since=datetime(2012, 3, 1, 23, 0, 0)),
            lambda g: g.id,
            ["2793505", "2793179", "11cb445f8197e17d303d"],
        )

    def testGetStarredGists(self):
        self.assertListKeyEqual(
            self.user.get_starred_gists(),
            lambda g: g.id,
            ["1942384", "dcb7de17e8a52b74541d"],
        )

    def testGetIssues(self):
        self.assertListKeyEqual(
            self.user.get_issues(),
            lambda i: (i.id, i.repository.name),
            [
                (4639931, "PyGithub"),
                (4452000, "PyGithub"),
                (4356743, "PyGithub"),
                (3716033, "PyGithub"),
                (3715946, "PyGithub"),
                (3643837, "PyGithub"),
                (3628022, "PyGithub"),
                (3624595, "PyGithub"),
                (3624570, "PyGithub"),
                (3624561, "PyGithub"),
                (3624556, "PyGithub"),
                (3619973, "PyGithub"),
                (3527266, "PyGithub"),
                (3527245, "PyGithub"),
                (3527231, "PyGithub"),
            ],
        )

    def testGetIssuesWithAllArguments(self):
        requestedByUser = self.user.get_repo("PyGithub").get_label("Requested by user")
        issues = self.user.get_issues(
            "assigned",
            "closed",
            [requestedByUser],
            "comments",
            "asc",
            datetime(2012, 5, 28, 23, 0, 0),
        )
        self.assertListKeyEqual(
            issues,
            lambda i: i.id,
            [
                6816576,
                8495415,
                6889934,
                8339699,
                8075253,
                8033963,
                9089893,
                9489725,
                11746141,
                5152384,
                5177381,
                5783131,
                6454054,
                6641076,
                6653907,
                7331214,
                9489813,
                9776615,
                10360280,
                4356743,
                6583381,
                6751469,
                8189836,
                10758585,
                12097154,
                12867103,
                5191621,
                5256315,
                6363719,
                9209408,
                6912733,
                9948505,
                11503771,
                10922412,
                11844658,
                12566144,
                6353712,
                9323084,
                10379143,
                5387373,
                12179668,
                6911794,
                11731917,
                6807542,
                6780606,
            ],
        )

    def testGetUserIssues(self):
        self.assertListKeyEqual(
            self.user.get_user_issues(),
            lambda i: i.id,
            [
                14447880,
                13505356,
                12541184,
                10586808,
                6741461,
                6741457,
                6727331,
                5641572,
            ],
        )

    def testGetUserIssuesWithAllArguments(self):
        requestedByUser = self.user.get_repo("PyGithub").get_label("Requested by user")
        issues = self.user.get_user_issues(
            "assigned",
            "closed",
            [requestedByUser],
            "comments",
            "asc",
            datetime(2012, 5, 28, 23, 0, 0),
        )
        self.assertListKeyEqual(
            issues,
            lambda i: i.id,
            [
                6816576,
                8495415,
                6889934,
                8339699,
                8075253,
                8033963,
                9089893,
                9489725,
                11746141,
                5152384,
                5177381,
                5783131,
                6454054,
                6641076,
                6653907,
                7331214,
                9489813,
                9776615,
                10360280,
                4356743,
                6583381,
                6751469,
                8189836,
                10758585,
                12097154,
                12867103,
                5191621,
                5256315,
                6363719,
                9209408,
                6912733,
                9948505,
                11503771,
                10922412,
                11844658,
                12566144,
                6353712,
                9323084,
                10379143,
                5387373,
                12179668,
                6911794,
                11731917,
                6807542,
                6780606,
            ],
        )

    def testGetKeys(self):
        self.assertListKeyEqual(
            self.user.get_keys(),
            lambda k: k.title,
            ["vincent@home", "vincent@gandi", "vincent@aws", "vincent@macbook"],
        )

    def testGetOrgs(self):
        self.assertListKeyEqual(self.user.get_orgs(), lambda o: o.login, ["BeaverSoftware"])

    def testGetRepos(self):
        self.assertListKeyEqual(
            self.user.get_repos(),
            lambda r: r.name,
            [
                "TestPyGithub",
                "django",
                "PyGithub",
                "developer.github.com",
                "acme-public-website",
                "C4Planner",
                "Hacking",
                "vincent-jacques.net",
                "Contests",
                "Candidates",
                "Tests",
                "DrawTurksHead",
                "DrawSyntax",
                "QuadProgMm",
                "Boost.HierarchicalEnum",
                "ViDE",
            ],
        )

    def testGetReposWithArguments(self):
        self.assertListKeyEqual(
            self.user.get_repos("all", "owner", "public", "full_name", "desc"),
            lambda r: r.name,
            [
                "ViDE",
                "QuadProgMm",
                "PyGithub",
                "DrawTurksHead",
                "DrawSyntax",
                "django",
                "developer.github.com",
                "C4Planner",
                "Boost.HierarchicalEnum",
                "acme-public-website",
            ],
        )

    def testCreateFork(self):
        repo = self.user.create_fork(self.g.get_user("nvie").get_repo("gitflow"))
        self.assertEqual(repo.source.full_name, "nvie/gitflow")

    def testCreateRepoFromTemplate(self):
        template_repo = self.g.get_repo("actions/hello-world-docker-action")

        repo = self.user.create_repo_from_template("hello-world-docker-action-new", template_repo)
        self.assertEqual(
            repo.url,
            "https://api.github.com/repos/jacquev6/hello-world-docker-action-new",
        )
        self.assertFalse(repo.is_template)

    def testCreateRepoFromTemplateWithAllArguments(self):
        template_repo = self.g.get_repo("actions/hello-world-docker-action")

        description = "My repo from template"
        private = True
        repo = self.user.create_repo_from_template(
            "hello-world-docker-action-new",
            template_repo,
            description=description,
            include_all_branches=True,
            private=private,
        )
        self.assertEqual(repo.description, description)
        self.assertTrue(repo.private)

    def testGetNotification(self):
        notification = self.user.get_notification("8406712")
        self.assertEqual(notification.id, "8406712")
        self.assertEqual(notification.unread, False)
        self.assertEqual(notification.reason, "author")
        self.assertEqual(notification.subject.title, "Feature/coveralls")
        self.assertEqual(notification.subject.type, "PullRequest")
        self.assertEqual(notification.repository.id, 8432784)
        self.assertEqual(
            notification.updated_at,
            datetime(2013, 3, 15, 5, 43, 11, tzinfo=timezone.utc),
        )
        self.assertEqual(notification.url, "/notifications/threads/8406712")
        self.assertEqual(notification.subject.url, None)
        self.assertEqual(notification.subject.latest_comment_url, None)
        self.assertEqual(
            repr(notification),
            'Notification(subject=NotificationSubject(title="Feature/coveralls"), id="8406712")',
        )
        self.assertEqual(repr(notification.subject), 'NotificationSubject(title="Feature/coveralls")')

    def testGetNotifications(self):
        self.assertListKeyEqual(self.user.get_notifications(participating=True), lambda n: n.id, ["8406712"])

    def testGetNotificationsWithOtherArguments(self):
        self.assertListKeyEqual(self.user.get_notifications(all=True), lambda n: n.id, [])

    def testMarkNotificationsAsRead(self):
        self.user.mark_notifications_as_read(datetime(2018, 10, 18, 18, 20, 0o1, 0))

    def testGetTeams(self):
        self.assertListKeyEqual(
            self.user.get_teams(),
            lambda t: t.name,
            [
                "Owners",
                "Honoraries",
                "Honoraries",
                "Honoraries",
                "Honoraries",
                "Honoraries",
                "Honoraries",
                "Honoraries",
                "Honoraries",
                "Honoraries",
            ],
        )

    def testAcceptInvitation(self):
        self.assertEqual(self.user.accept_invitation(4294886), None)

    def testGetInvitations(self):
        invitation = self.user.get_invitations()[0]
        self.assertEqual(repr(invitation), "Invitation(id=17285388)")
        self.assertEqual(invitation.id, 17285388)
        self.assertEqual(invitation.permissions, "write")
        created_at = datetime(2019, 6, 27, 11, 47, tzinfo=timezone.utc)
        self.assertEqual(invitation.created_at, created_at)
        self.assertEqual(invitation.expired, True)
        self.assertEqual(
            invitation.url,
            "https://api.github.com/user/repository_invitations/17285388",
        )
        self.assertEqual(invitation.html_url, "https://github.com/jacquev6/PyGithub/invitations")
        self.assertEqual(invitation.node_id, "MDIwOlJlcG9zaXRvcnlJbnZpdGF0aW9uMTcyODUzODg=")
        self.assertEqual(invitation.repository.name, "PyGithub")
        self.assertEqual(invitation.invitee.login, "foobar-test1")
        self.assertEqual(invitation.inviter.login, "jacquev6")

    def testCreateMigration(self):
        self.assertTrue(isinstance(self.user.create_migration(["sample-repo"]), github.Migration.Migration))

    def testGetMigrations(self):
        self.assertEqual(self.user.get_migrations().totalCount, 46)

    def testInstallations(self):
        installations = self.user.get_installations()
        self.assertEqual(installations[0].id, 123456)
        self.assertEqual(installations[0].app_id, 10101)
        self.assertEqual(installations[0].target_id, 3344556)
        self.assertEqual(installations[0].target_type, "User")
        self.assertEqual(installations.totalCount, 1)

    def testGetMemberships(self):
        membership_data = self.user.get_organization_memberships()
        self.assertListKeyEqual(
            membership_data,
            lambda e: e.organization.login,
            ["aneyem-github", "nko4", "geoservel", "iic2154-uc-cl", "nnodes", "sushiclm"],
        )
        self.assertListKeyEqual(
            membership_data, lambda e: e.role, ["member", "member", "admin", "member", "member", "admin"]
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

from . import Framework


class RepositoryDiscussion(Framework.TestCase):
    discussion_schema = """
        answer {
          author { login }
          body
          bodyHTML
          bodyText
          createdAt
          databaseId
          discussion { id }
          editor { login }
          id
          lastEditedAt
          updatedAt
          url
        }
        author { login }
        body
        bodyHTML
        bodyText
        category {
          createdAt
          description
          emoji
          emojiHTML
          id
          isAnswerable
          name
          repository { owner { login } name }
          slug
          updatedAt
        }
        comments(first: 10) {
          totalCount
          pageInfo {
            startCursor
            endCursor
            hasNextPage
            hasPreviousPage
          }
          nodes {
            id
          }
        }
        createdAt
        databaseId
        editor { login }
        id
        labels(first: 10) {
          totalCount
          pageInfo {
            startCursor
            endCursor
            hasNextPage
            hasPreviousPage
          }
          nodes {
            id
            name
          }
        }
        lastEditedAt
        number
        reactions(first: 10) {
          totalCount
          pageInfo {
            startCursor
            endCursor
            hasNextPage
            hasPreviousPage
          }
          nodes {
            id
          }
        }
        repository {
          owner { login }
          name
        }
        title
        updatedAt
        url
      """

    def setUp(self):
        super().setUp()
        self.discussion = self.g.get_repository_discussion("D_kwDOADYVqs4AaHoG", self.discussion_schema)

    def testAttributes(self):
        self.assertEqual(self.discussion.answer.author.login, "dawngerpony")
        self.assertEqual(
            self.discussion.answer.body,
            """[This comment](https://github.com/PyGithub/PyGithub/issues/2895#issue-2118964108) contains the answer to your question:\r\n\r\n```python\r\nmy_repo.raw_data["custom_properties"]\r\n```\r\n\r\n#2968 appears to add proper support for custom properties, but doesn't look like it's made it into a release yet.""",
        )
        self.assertEqual(
            self.discussion.answer.body_html,
            """<p dir="auto"><a href="https://github.com/PyGithub/PyGithub/issues/2895#issue-2118964108" data-hovercard-type="issue" data-hovercard-url="/PyGithub/PyGithub/issues/2895/hovercard">This comment</a> contains the answer to your question:</p>\n<div class="highlight highlight-source-python notranslate position-relative overflow-auto" dir="auto" data-snippet-clipboard-copy-content="my_repo.raw_data[&quot;custom_properties&quot;]"><pre class="notranslate"><span class="pl-s1">my_repo</span>.<span class="pl-s1">raw_data</span>[<span class="pl-s">"custom_properties"</span>]</pre></div>\n<p dir="auto"><a class="issue-link js-issue-link" data-error-text="Failed to load title" data-id="2293897055" data-permission-text="Title is private" data-url="https://github.com/PyGithub/PyGithub/issues/2968" data-hovercard-type="pull_request" data-hovercard-url="/PyGithub/PyGithub/pull/2968/hovercard" href="https://github.com/PyGithub/PyGithub/pull/2968">#2968</a> appears to add proper support for custom properties, but doesn't look like it's made it into a release yet.</p>""",
        )
        self.assertEqual(
            self.discussion.answer.body_text,
            "This comment contains the answer to your question:\nmy_repo.raw_data[\"custom_properties\"]\n#2968 appears to add proper support for custom properties, but doesn't look like it's made it into a release yet.",
        )
        self.assertEqual(self.discussion.answer.created_at, datetime(2024, 8, 23, 6, 36, 50, tzinfo=timezone.utc))
        self.assertEqual(self.discussion.answer.database_id, 10426644)
        self.assertEqual(self.discussion.answer.discussion.node_id, "D_kwDOADYVqs4AaHoG")
        self.assertIsNone(self.discussion.answer.editor)
        self.assertEqual(self.discussion.answer.node_id, "DC_kwDOADYVqs4AnxkU")
        self.assertIsNone(self.discussion.answer.last_edited_at)
        self.assertEqual(self.discussion.answer.updated_at, datetime(2024, 8, 23, 6, 36, 51, tzinfo=timezone.utc))
        self.assertEqual(
            self.discussion.answer.html_url,
            "https://github.com/PyGithub/PyGithub/discussions/2993#discussioncomment-10426644",
        )
        self.assertEqual(self.discussion.author.login, "heitorPB")
        self.assertEqual(
            self.discussion.body,
            """What is the equivalent of `https://api.github.com/repos/OWNER/REPO/properties/values`? I'm interested in getting/setting custom properties for my repos. I can do that with `curl`, but couldn't find a way to do it via this project.\r\n\r\nDocs here: [Get all custom property values for a repository](https://docs.github.com/en/rest/repos/custom-properties?apiVersion=2022-11-28#get-all-custom-property-values-for-a-repository).""",
        )
        self.assertEqual(
            self.discussion.body_html,
            """<p dir="auto">What is the equivalent of <code class="notranslate">https://api.github.com/repos/OWNER/REPO/properties/values</code>? I'm interested in getting/setting custom properties for my repos. I can do that with <code class="notranslate">curl</code>, but couldn't find a way to do it via this project.</p>\n<p dir="auto">Docs here: <a href="https://docs.github.com/en/rest/repos/custom-properties?apiVersion=2022-11-28#get-all-custom-property-values-for-a-repository">Get all custom property values for a repository</a>.</p>""",
        )
        self.assertEqual(
            self.discussion.body_text,
            """What is the equivalent of https://api.github.com/repos/OWNER/REPO/properties/values? I'm interested in getting/setting custom properties for my repos. I can do that with curl, but couldn't find a way to do it via this project.\nDocs here: Get all custom property values for a repository.""",
        )
        self.assertEqual(self.discussion.category.created_at, datetime(2020, 12, 8, 23, 29, 13, tzinfo=timezone.utc))
        self.assertEqual(self.discussion.category.description, "Ask the community for help")
        self.assertEqual(self.discussion.category.emoji, ":pray:")
        self.assertEqual(self.discussion.category.emoji_html, "<div>🙏</div>")
        self.assertEqual(self.discussion.category.id, "MDE4OkRpc2N1c3Npb25DYXRlZ29yeTMyMDI5MDYx")
        self.assertEqual(self.discussion.category.is_answerable, True)
        self.assertEqual(self.discussion.category.name, "Q&A")
        self.assertEqual(self.discussion.category.repository.owner.login, "PyGithub")
        self.assertEqual(self.discussion.category.repository.name, "PyGithub")
        self.assertEqual(self.discussion.category.slug, "q-a")
        self.assertEqual(self.discussion.category.updated_at, datetime(2020, 12, 8, 23, 29, 13, tzinfo=timezone.utc))
        self.assertEqual(self.discussion.created_at, datetime(2024, 6, 21, 13, 11, 38, tzinfo=timezone.utc))
        self.assertEqual(self.discussion.database_id, 6846982)
        self.assertIsNone(self.discussion.editor)
        self.assertEqual(self.discussion.node_id, "D_kwDOADYVqs4AaHoG")
        self.assertIsNone(self.discussion.last_edited_at)
        self.assertEqual(self.discussion.number, 2993)
        self.assertEqual(self.discussion.repository.owner.login, "PyGithub")
        self.assertEqual(self.discussion.repository.name, "PyGithub")
        self.assertEqual(self.discussion.title, "How to get a list of custom repository properties?")
        self.assertEqual(self.discussion.updated_at, datetime(2024, 8, 29, 16, 1, 0, tzinfo=timezone.utc))

        self.assertListEqual(
            [c.node_id for c in self.discussion.get_comments("id")], ["DC_kwDOADYVqs4AnxkU", "DC_kwDOADYVqs4AoA2V"]
        )
        self.assertEqual(self.discussion.get_labels().totalCount, 0)
        self.assertEqual(self.discussion.get_reactions().totalCount, 0)

    def testGetComments(self):
        discussion = self.g.get_repository_discussion("D_kwDOADYVqs4AaHoG", "id")
        comments_pages = discussion.get_comments("id")
        comments = list(comments_pages)
        self.assertEqual(comments_pages.totalCount, 2)
        self.assertEqual(len(comments), 2)
        self.assertListEqual([c.id for c in comments], ["DC_kwDOADYVqs4AnxkU", "DC_kwDOADYVqs4AoA2V"])

    def testGetCommentsWithoutNodeId(self):
        discussion = self.g.get_repository_discussion("D_kwDOADYVqs4AaHoG", "title")
        with self.assertRaises(RuntimeError) as e:
            discussion.get_comments("id")
        self.assertEqual(e.exception.args, ("Retrieving discussion comments requires the discussion field 'id'",))

    def testAddAndDeleteComment(self):
        discussion = self.g.get_repository_discussion("D_kwDOADYVqs4AaHoG", "id")
        comment = discussion.add_comment("test comment", output_schema="id body")
        self.assertEqual(comment.id, "DC_kwDOADYVqs4AovYk")
        self.assertEqual(comment.body, "test comment")

        reply = discussion.add_comment("test reply", reply_to=comment, output_schema="id body")
        self.assertEqual(reply.id, "DC_kwDOADYVqs4AovYl")
        self.assertEqual(reply.body, "test reply")

        reply_by_id_str = discussion.add_comment(
            "test reply by string id", reply_to=comment.id, output_schema="id body"
        )
        self.assertEqual(reply_by_id_str.id, "DC_kwDOADYVqs4AovYm")
        self.assertEqual(reply_by_id_str.body, "test reply by string id")

        # cleanup that discussion
        # replies have to be deleted first to be able to fully delete a comment
        self.assertEqual(reply_by_id_str.delete().id, "DC_kwDOADYVqs4AovYm")
        self.assertEqual(reply.delete().id, "DC_kwDOADYVqs4AovYl")
        self.assertEqual(comment.delete().id, "DC_kwDOADYVqs4AovYk")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Balázs Rostás <rostas.balazs@gmail.com>                       #
# Copyright 2018 Anton Nguyen <afnguyen85@gmail.com>                           #
# Copyright 2018 Jacopo Notarstefano <jacopo.notarstefano@gmail.com>           #
# Copyright 2018 Jasper van Wanrooy <jasper@vanwanrooy.net>                    #
# Copyright 2018 Raihaan <31362124+res0nance@users.noreply.github.com>         #
# Copyright 2018 Shubham Singh <41840111+singh811@users.noreply.github.com>    #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Tim Boring <tboring@hearst.com>                               #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Brian Choy <byceee@gmail.com>                                 #
# Copyright 2019 Geoffroy Jabouley <gjabouley@invensense.com>                  #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2019 ebrown <brownierin@users.noreply.github.com>                  #
# Copyright 2020 Geoff Low <glow@mdsol.com>                                    #
# Copyright 2020 Glenn McDonald <testworksau@users.noreply.github.com>         #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2020 latacora-daniel <71085674+latacora-daniel@users.noreply.github.com>#
# Copyright 2020 ton-katsu <sakamoto.yoshihisa@gmail.com>                      #
# Copyright 2021 Marina Peresypkina <mi9onev@gmail.com>                        #
# Copyright 2021 Tanner <51724788+lightningboltemoji@users.noreply.github.com> #
# Copyright 2022 KimSia Sim <245021+simkimsia@users.noreply.github.com>        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Felipe Peter <mr-peipei@web.de>                               #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Jonathan Greg <31892308+jmgreg31@users.noreply.github.com>    #
# Copyright 2023 Mauricio Alejandro Martínez Pacheco <mauricio.martinez@premise.com>#
# Copyright 2023 Mauricio Alejandro Martínez Pacheco <n_othing@hotmail.com>    #
# Copyright 2024 Andrii Kezikov <cheshirez@gmail.com>                          #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Jacky Lam <jacky.lam@r2studiohk.com>                          #
# Copyright 2024 Mohamed Mostafa <112487260+mohy01@users.noreply.github.com>   #
# Copyright 2024 Oskar Jansson <56458534+janssonoskar@users.noreply.github.com>#
# Copyright 2024 Thomas Cooper <coopernetes@proton.me>                         #
# Copyright 2024 Thomas Crowley <15927917+thomascrowley@users.noreply.github.com>#
# Copyright 2025 Bill Napier <napier@pobox.com>                                #
# Copyright 2025 Dom Heinzeller <dom.heinzeller@icloud.com>                    #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Greg Fogelberg <52933995+gfog-floqast@users.noreply.github.com>#
# Copyright 2025 Pavel Abramov <31950564+uncleDecart@users.noreply.github.com> #
# Copyright 2025 Zachary <6599715+interifter@users.noreply.github.com>         #
# Copyright 2026 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timedelta, timezone
from unittest import mock

import github
from github.OrganizationCustomProperty import CustomProperty

from . import Framework


class Organization(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.org = self.g.get_organization("BeaverSoftware")

    def testAttributes(self):
        self.assertIsNone(self.org.advanced_security_enabled_for_new_repositories)
        self.assertIsNone(self.org.archived_at)
        self.assertEqual(self.org.avatar_url, "https://avatars.githubusercontent.com/u/12345678?v=4")
        self.assertEqual(self.org.billing_email, "foo@example.com")
        self.assertEqual(self.org.blog, "http://www.example.com")
        self.assertEqual(self.org.collaborators, 9)
        self.assertEqual(self.org.company, None)
        self.assertEqual(self.org.created_at, datetime(2014, 1, 9, 16, 56, 17, tzinfo=timezone.utc))
        self.assertIsNone(self.org.default_repository_branch)
        self.assertIsNone(self.org.default_repository_permission)
        self.assertIsNone(self.org.dependabot_alerts_enabled_for_new_repositories)
        self.assertIsNone(self.org.dependabot_security_updates_enabled_for_new_repositories)
        self.assertIsNone(self.org.dependency_graph_enabled_for_new_repositories)
        self.assertEqual(self.org.deploy_keys_enabled_for_repositories, True)
        self.assertEqual(self.org.description, "BeaverSoftware writes software.")
        self.assertIsNone(self.org.disk_usage)
        self.assertIsNone(self.org.display_commenter_full_name_setting_enabled)
        self.assertIsNone(self.org.display_login)
        self.assertEqual(self.org.email, "foo@example.com")
        self.assertEqual(self.org.events_url, "https://api.github.com/orgs/BeaverSoftware/events")
        self.assertEqual(self.org.followers, 130)
        self.assertIsNone(self.org.followers_url)
        self.assertEqual(self.org.following, 1)
        self.assertIsNone(self.org.following_url)
        self.assertIsNone(self.org.gists_url)
        self.assertEqual(self.org.gravatar_id, None)
        self.assertEqual(self.org.has_organization_projects, True)
        self.assertEqual(self.org.has_repository_projects, True)
        self.assertEqual(self.org.hooks_url, "https://api.github.com/orgs/BeaverSoftware/hooks")
        self.assertEqual(self.org.html_url, "https://github.com/BeaverSoftware")
        self.assertEqual(self.org.id, 21341965)
        self.assertEqual(self.org.is_verified, False)
        self.assertEqual(self.org.issues_url, "https://api.github.com/orgs/BeaverSoftware/issues")
        self.assertEqual(self.org.location, "Paris, France")
        self.assertEqual(self.org.login, "BeaverSoftware")
        self.assertEqual(self.org.members_allowed_repository_creation_type, "none")
        self.assertIsNone(self.org.members_can_change_repo_visibility)
        self.assertEqual(self.org.members_can_create_internal_repositories, False)
        self.assertEqual(self.org.members_can_create_pages, True)
        self.assertEqual(self.org.members_can_create_private_pages, True)
        self.assertEqual(self.org.members_can_create_private_repositories, False)
        self.assertEqual(self.org.members_can_create_public_pages, True)
        self.assertEqual(self.org.members_can_create_public_repositories, False)
        self.assertEqual(self.org.members_can_create_repositories, False)
        self.assertIsNone(self.org.members_can_create_teams)
        self.assertIsNone(self.org.members_can_delete_issues)
        self.assertIsNone(self.org.members_can_delete_repositories)
        self.assertEqual(self.org.members_can_fork_private_repositories, False)
        self.assertIsNone(self.org.members_can_invite_outside_collaborators)
        self.assertIsNone(self.org.members_can_view_dependency_insights)
        self.assertEqual(self.org.members_url, "https://api.github.com/orgs/BeaverSoftware/members{/member}")
        self.assertEqual(self.org.name, "BeaverSoftware")
        self.assertEqual(self.org.node_id, "AbCdEfG")
        self.assertIsNone(self.org.organizations_url)
        self.assertEqual(self.org.owned_private_repos, 191)
        self.assertEqual(self.org.plan.name, "enterprise")
        self.assertEqual(self.org.plan.private_repos, 999999)
        self.assertEqual(self.org.plan.space, 123456789)
        self.assertEqual(self.org.plan.filled_seats, 640)
        self.assertEqual(self.org.plan.seats, 1024)
        self.assertIsNone(self.org.private_gists)
        self.assertEqual(self.org.public_gists, 0)
        self.assertEqual(
            self.org.public_members_url, "https://api.github.com/orgs/BeaverSoftware/public_members{/member}"
        )
        self.assertEqual(self.org.public_repos, 121)
        self.assertIsNone(self.org.readers_can_create_discussions)
        self.assertIsNone(self.org.received_events_url)
        self.assertEqual(self.org.repos_url, "https://api.github.com/orgs/BeaverSoftware/repos")
        self.assertIsNone(self.org.secret_scanning_enabled_for_new_repositories)
        self.assertIsNone(self.org.secret_scanning_push_protection_custom_link)
        self.assertIsNone(self.org.secret_scanning_push_protection_custom_link_enabled)
        self.assertIsNone(self.org.secret_scanning_push_protection_enabled_for_new_repositories)
        self.assertIsNone(self.org.site_admin)
        self.assertIsNone(self.org.starred_at)
        self.assertIsNone(self.org.starred_url)
        self.assertIsNone(self.org.subscriptions_url)
        self.assertEqual(self.org.total_private_repos, 176)
        self.assertIsNone(self.org.twitter_username)
        self.assertEqual(self.org.two_factor_requirement_enabled, True)
        self.assertEqual(self.org.type, "Organization")
        self.assertEqual(self.org.updated_at, datetime(2024, 8, 20, 8, 44, 26, tzinfo=timezone.utc))
        self.assertEqual(self.org.url, "https://api.github.com/orgs/BeaverSoftware")
        self.assertEqual(repr(self.org), 'Organization(login="BeaverSoftware")')
        self.assertIsNone(self.org.user_view_type)
        self.assertEqual(self.org.web_commit_signoff_required, False)

    def testLazyAttributes(self):
        org = self.g.withLazy(True).get_organization("org")
        self.assertEqual(org.login, "org")
        self.assertEqual(org.url, "/orgs/org")

    def testAddMembersDefaultRole(self):
        lyloa = self.g.get_user("lyloa")
        self.assertFalse(self.org.has_in_members(lyloa))
        self.org.add_to_members(lyloa, role="member")
        # 'Pending' members won't be in /orgs/:org/members/:user
        self.assertFalse(self.org.has_in_members(lyloa))
        self.org.remove_from_membership(lyloa)
        self.assertFalse(self.org.has_in_members(lyloa))

    def testAddMembersAdminRole(self):
        lyloa = self.g.get_user("lyloa")
        self.assertFalse(self.org.has_in_members(lyloa))
        self.org.add_to_members(lyloa, role="admin")
        # 'Pending' members won't be in /orgs/:org/members/:user
        self.assertFalse(self.org.has_in_members(lyloa))
        self.org.remove_from_membership(lyloa)
        self.assertFalse(self.org.has_in_members(lyloa))

    def testEditWithoutArguments(self):
        self.org.edit()

    def testEditWithAllArguments(self):
        self.org.edit(
            "BeaverSoftware2@vincent-jacques.net",
            "http://vincent-jacques.net",
            "Company edited by PyGithub",
            "Description edited by PyGithub",
            "BeaverSoftware2@vincent-jacques.net",
            "Location edited by PyGithub",
            "Name edited by PyGithub",
        )
        self.assertEqual(self.org.billing_email, "BeaverSoftware2@vincent-jacques.net")
        self.assertEqual(self.org.blog, "http://vincent-jacques.net")
        self.assertEqual(self.org.company, "Company edited by PyGithub")
        self.assertEqual(self.org.description, "Description edited by PyGithub")
        self.assertEqual(self.org.email, "BeaverSoftware2@vincent-jacques.net")
        self.assertEqual(self.org.location, "Location edited by PyGithub")
        self.assertEqual(self.org.name, "Name edited by PyGithub")

    def testEditHookWithMinimalParameters(self):
        hook = self.org.create_hook("web", {"url": "http://foobar.com"})
        hook = self.org.edit_hook(hook.id, "mobile", {"url": "http://barfoo.com"})
        self.assertEqual(hook.name, "mobile")

    def testEditHookWithAllParameters(self):
        hook = self.org.create_hook("web", {"url": "http://foobar.com"}, ["fork"], False)
        hook = self.org.edit_hook(hook.id, "mobile", {"url": "http://barfoo.com"}, ["spoon"], True)
        self.assertEqual(hook.name, "mobile")
        self.assertEqual(hook.events, ["spoon"])
        self.assertEqual(hook.active, True)

    def testCreateTeam(self):
        team = self.org.create_team("Team created by PyGithub")
        self.assertEqual(team.id, 189850)

    def testCreateTeamWithAllArguments(self):
        repo = self.org.get_repo("FatherBeaver")
        parent_team = self.org.get_team(141496)
        maintainer = self.g.get_user("jacquev6")
        team = self.org.create_team(
            "Team also created by PyGithub",
            [repo],
            "push",
            "secret",
            "Description also created by PyGithub",
            parent_team.id,
            [maintainer.login],
            "notifications_disabled",
        )
        self.assertEqual(team.id, 189852)
        self.assertEqual(team.description, "Description also created by PyGithub")
        self.assertEqual(team.parent, parent_team)
        self.assertEqual(team.notification_setting, "notifications_disabled")
        self.assertEqual(maintainer.login, team.get_members(role="maintainer")[0].name)

    def testDeleteHook(self):
        hook = self.org.create_hook("web", {"url": "http://foobar.com"})
        self.org.delete_hook(hook.id)

    def testPublicMembers(self):
        lyloa = self.g.get_user("Lyloa")
        self.assertFalse(self.org.has_in_public_members(lyloa))
        self.org.add_to_public_members(lyloa)
        self.assertTrue(self.org.has_in_public_members(lyloa))
        self.org.remove_from_public_members(lyloa)
        self.assertFalse(self.org.has_in_public_members(lyloa))

    def testGetPublicMembers(self):
        self.assertListKeyEqual(self.org.get_public_members(), lambda u: u.login, ["jacquev6"])

    def testGetHook(self):
        hook = self.org.get_hook(257993)
        self.assertEqual(hook.name, "web")

    def testGetHooks(self):
        self.assertListKeyEqual(self.org.get_hooks(), lambda h: h.id, [257993])

    def testGetHookDelivery(self):
        delivery = self.org.get_hook_delivery(257993, 12345)
        self.assertEqual(delivery.id, 12345)
        self.assertEqual(delivery.guid, "abcde-12345")
        self.assertEqual(
            delivery.delivered_at,
            datetime(2012, 5, 27, 6, 0, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(delivery.redelivery, False)
        self.assertEqual(delivery.duration, 0.27)
        self.assertEqual(delivery.status, "OK")
        self.assertEqual(delivery.status_code, 200)
        self.assertIsNone(delivery.throttled_at)
        self.assertEqual(delivery.event, "issues")
        self.assertEqual(delivery.action, "opened")
        self.assertEqual(delivery.installation_id, 123)
        self.assertEqual(delivery.repository_id, 456)
        self.assertEqual(delivery.url, "https://www.example-webhook.com")
        self.assertIsInstance(delivery.request, github.HookDelivery.HookDeliveryRequest)
        self.assertEqual(delivery.request.headers, {"content-type": "application/json"})
        self.assertEqual(delivery.request.payload, {"action": "opened"})
        self.assertIsInstance(delivery.response, github.HookDelivery.HookDeliveryResponse)
        self.assertEqual(delivery.response.headers, {"content-type": "text/html;charset=utf-8"})
        self.assertEqual(delivery.response.payload, "ok")

    def testGetHookDeliveries(self):
        deliveries = list(self.org.get_hook_deliveries(257993))
        self.assertEqual(len(deliveries), 1)
        self.assertEqual(deliveries[0].id, 12345)
        self.assertEqual(deliveries[0].guid, "abcde-12345")
        self.assertEqual(
            deliveries[0].delivered_at,
            datetime(2012, 5, 27, 6, 0, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(deliveries[0].redelivery, False)
        self.assertEqual(deliveries[0].duration, 0.27)
        self.assertEqual(deliveries[0].status, "OK")
        self.assertEqual(deliveries[0].status_code, 200)
        self.assertEqual(deliveries[0].event, "issues")
        self.assertEqual(deliveries[0].action, "opened")
        self.assertEqual(deliveries[0].installation_id, 123)
        self.assertEqual(deliveries[0].repository_id, 456)
        self.assertEqual(deliveries[0].url, "https://www.example-webhook.com")

    def testGetIssues(self):
        self.assertListKeyEqual(self.org.get_issues(), lambda i: i.id, [])

    def testGetIssuesWithAllArguments(self):
        requestedByUser = self.g.get_user().get_repo("PyGithub").get_label("Requested by user")
        issues = self.org.get_issues(
            "assigned",
            "closed",
            [requestedByUser],
            "comments",
            "asc",
            datetime(2012, 5, 28, 23, 0, 0, tzinfo=timezone.utc),
        )
        self.assertListKeyEqual(issues, lambda i: i.id, [])

    def testGetMembers(self):
        self.assertListKeyEqual(self.org.get_members(), lambda u: u.login, ["cjuniet", "jacquev6", "Lyloa"])

    def testGetOutsideCollaborators(self):
        self.assertListKeyEqual(self.org.get_outside_collaborators(), lambda u: u.login, ["octocat"])

    def testOutsideCollaborators(self):
        octocat = self.g.get_user("octocat")
        self.org.convert_to_outside_collaborator(octocat)
        self.assertListKeyEqual(self.org.get_outside_collaborators(), lambda u: u.login, ["octocat"])
        self.org.remove_outside_collaborator(octocat)
        self.assertEqual(list(self.org.get_outside_collaborators()), [])

    def testMembers(self):
        lyloa = self.g.get_user("Lyloa")
        self.assertTrue(self.org.has_in_members(lyloa))
        self.org.remove_from_members(lyloa)
        self.assertFalse(self.org.has_in_members(lyloa))

    def testGetRepos(self):
        repos = self.org.get_repos()
        self.assertListKeyEqual(repos, lambda r: r.name, ["FatherBeaver", "TestPyGithub"])
        self.assertListKeyEqual(repos, lambda r: r.has_pages, [True, False])
        self.assertListKeyEqual(repos, lambda r: r.has_wiki, [True, True])

    def testGetReposSorted(self):
        repos = self.org.get_repos(sort="updated", direction="desc")
        self.assertListKeyEqual(
            repos,
            lambda r: r.name,
            ["TestPyGithub", "FatherBeaver"],
        )
        self.assertListKeyEqual(
            repos,
            lambda r: r.has_pages,
            [False, True],
        )

    def testGetReposWithType(self):
        repos = self.org.get_repos("public")
        self.assertListKeyEqual(repos, lambda r: r.name, ["FatherBeaver", "PyGithub"])
        self.assertListKeyEqual(repos, lambda r: r.has_pages, [True, True])

    def testGetEvents(self):
        self.assertListKeyEqual(
            self.org.get_events(),
            lambda e: e.type,
            [
                "CreateEvent",
                "CreateEvent",
                "PushEvent",
                "PushEvent",
                "DeleteEvent",
                "DeleteEvent",
                "PushEvent",
                "PushEvent",
                "DeleteEvent",
                "DeleteEvent",
                "PushEvent",
                "PushEvent",
                "PushEvent",
                "CreateEvent",
                "CreateEvent",
                "CreateEvent",
                "CreateEvent",
                "CreateEvent",
                "PushEvent",
                "PushEvent",
                "PushEvent",
                "PushEvent",
                "PushEvent",
                "PushEvent",
                "ForkEvent",
                "CreateEvent",
            ],
        )

    def testGetTeams(self):
        self.assertListKeyEqual(self.org.get_teams(), lambda t: t.name, ["Members", "Owners"])

    def testGetTeamBySlug(self):
        team = self.org.get_team_by_slug("Members")
        self.assertEqual(team.id, 141496)

    def testCreateHookWithMinimalParameters(self):
        hook = self.org.create_hook("web", {"url": "http://foobar.com"})
        self.assertEqual(hook.id, 257967)

    def testCreateHookWithAllParameters(self):
        hook = self.org.create_hook("web", {"url": "http://foobar.com"}, ["fork"], False)
        self.assertTrue(hook.active)
        self.assertEqual(hook.id, 257993)

    def testCreateRepoWithMinimalArguments(self):
        repo = self.org.create_repo(name="TestPyGithub")
        self.assertEqual(repo.url, "https://api.github.com/repos/BeaverSoftware/TestPyGithub")
        self.assertTrue(repo.has_wiki)
        self.assertTrue(repo.has_pages)

    def testCreateRepoWithAllArguments(self):
        team = self.org.get_team(141496)
        repo = self.org.create_repo(
            name="TestPyGithub2",
            description="Repo created by PyGithub",
            homepage="http://foobar.com",
            private=False,
            visibility="public",
            has_issues=False,
            has_projects=False,
            has_wiki=False,
            has_downloads=False,
            team_id=team.id,
            allow_update_branch=True,
            allow_squash_merge=False,
            allow_merge_commit=False,
            allow_rebase_merge=True,
            delete_branch_on_merge=False,
        )
        self.assertEqual(repo.url, "https://api.github.com/repos/BeaverSoftware/TestPyGithub2")
        self.assertTrue(repo.allow_update_branch)
        self.assertFalse(repo.has_wiki)
        self.assertFalse(repo.has_pages)

    def testCreateRepositoryWithAutoInit(self):
        repo = self.org.create_repo(name="TestPyGithub", auto_init=True, gitignore_template="Python")
        self.assertEqual(repo.url, "https://api.github.com/repos/BeaverSoftware/TestPyGithub")
        self.assertTrue(repo.has_pages)
        self.assertTrue(repo.has_wiki)

    def testCreateFork(self):
        pygithub = self.g.get_user("jacquev6").get_repo("PyGithub")
        repo = self.org.create_fork(pygithub)
        self.assertEqual(repo.url, "https://api.github.com/repos/BeaverSoftware/PyGithub")
        self.assertFalse(repo.has_wiki)
        self.assertFalse(repo.has_pages)

    def testCreateRepoFromTemplate(self):
        template_repo = self.g.get_repo("actions/hello-world-docker-action")

        repo = self.org.create_repo_from_template("hello-world-docker-action-new", template_repo)
        self.assertEqual(
            repo.url,
            "https://api.github.com/repos/BeaverSoftware/hello-world-docker-action-new",
        )
        self.assertFalse(repo.is_template)

    def testCreateRepoFromTemplateWithAllArguments(self):
        template_repo = self.g.get_repo("actions/hello-world-docker-action")

        description = "My repo from template"
        private = True
        repo = self.org.create_repo_from_template(
            "hello-world-docker-action-new",
            template_repo,
            description=description,
            include_all_branches=True,
            private=private,
        )
        self.assertEqual(repo.description, description)
        self.assertTrue(repo.private)

    @mock.patch("github.PublicKey.encrypt")
    def testCreateSecretSelected(self, encrypt):
        repos = [self.org.get_repo("TestPyGithub"), self.org.get_repo("FatherBeaver")]
        # encrypt returns a non-deterministic value, we need to mock it so the replay data matches
        encrypt.return_value = "M+5Fm/BqTfB90h3nC7F3BoZuu3nXs+/KtpXwxm9gG211tbRo0F5UiN0OIfYT83CKcx9oKES9Va4E96/b"
        secret = self.org.create_secret(
            secret_name="secret-name",
            unencrypted_value="secret-value",
            visibility="selected",
            secret_type="actions",
            selected_repositories=repos,
        )

        self.assertIsNotNone(secret)
        self.assertEqual(secret.visibility, "selected")
        self.assertEqual(list(secret.selected_repositories), repos)

    def testGetSecret(self):
        repos = [self.org.get_repo("TestPyGithub"), self.org.get_repo("FatherBeaver")]
        secret = self.org.get_secret("secret-name")
        self.assertEqual(secret.name, "secret-name")
        self.assertEqual(secret.created_at, datetime(2019, 8, 10, 14, 59, 22, tzinfo=timezone.utc))
        self.assertEqual(secret.updated_at, datetime(2020, 1, 10, 14, 59, 22, tzinfo=timezone.utc))
        self.assertEqual(secret.visibility, "selected")
        self.assertEqual(list(secret.selected_repositories), repos)
        self.assertEqual(secret.url, "https://api.github.com/orgs/BeaverSoftware/actions/secrets/secret-name")

    def testLazySecret(self):
        secret = self.g.withLazy(True).get_organization("org").get_secret("secret name")
        self.assertEqual(str(secret), 'OrganizationSecret(name="secret name")')
        self.assertEqual(secret.name, "secret name")
        self.assertEqual(secret.url, "/orgs/org/actions/secrets/secret%20name")

    def testGetSecrets(self):
        secrets = self.org.get_secrets()
        self.assertEqual(len(list(secrets)), 1)

    def testGetDependabotSecrets(self):
        secrets = self.org.get_secrets(secret_type="dependabot")
        self.assertEqual(len(list(secrets)), 1)

    def testGetDependabotAlerts(self):
        alerts = self.org.get_dependabot_alerts()
        alert_list = list(alerts)
        self.assertEqual(len(list(alerts)), 8)
        self.assertEqual(alert_list[0].number, 1)
        self.assertEqual(alert_list[0].repository.full_name, "BeaverSoftware/PyGithub")

    def testGetDependabotAlertsWithAllArguments(self):
        alerts = self.org.get_dependabot_alerts(
            "open",
            "medium",
            "pip",
            "jinja2",
            "runtime",
            "updated",
            "asc",
        )
        alert_list = list(alerts)
        self.assertEqual(len(list(alerts)), 1)
        self.assertEqual(alert_list[0].number, 1)
        self.assertEqual(alert_list[0].state, "open")
        self.assertEqual(alert_list[0].security_advisory.severity, "medium")
        self.assertEqual(alert_list[0].dependency.package.ecosystem, "pip")
        self.assertEqual(alert_list[0].dependency.package.name, "jinja2")
        self.assertEqual(alert_list[0].dependency.scope, "runtime")
        self.assertEqual(alert_list[0].repository.full_name, "BeaverSoftware/PyGithub")

    def testGetSecretsFail(self):
        with self.assertRaises(AssertionError) as raisedexp:
            self.org.get_secrets(secret_type="secret")
        self.assertEqual("secret_type should be actions or dependabot", str(raisedexp.exception))

    def testInviteUserWithNeither(self):
        with self.assertRaises(AssertionError) as raisedexp:
            self.org.invite_user()
        self.assertEqual("specify only one of email or user", str(raisedexp.exception))

    def testInviteUserWithBoth(self):
        jacquev6 = self.g.get_user("jacquev6")
        with self.assertRaises(AssertionError) as raisedexp:
            self.org.invite_user(email="foo", user=jacquev6)
        self.assertEqual("specify only one of email or user", str(raisedexp.exception))

    def testInviteUserByName(self):
        jacquev6 = self.g.get_user("jacquev6")
        self.org.invite_user(user=jacquev6)

    def testInviteUserByEmail(self):
        self.org.invite_user(email="foo@example.com")

    def testInviteUserWithRoleAndTeam(self):
        team = self.org.create_team("Team created by PyGithub")
        self.org.invite_user(email="foo@example.com", role="billing_manager", teams=[team])

    def testInviteUserAsNonOwner(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.org.invite_user(email="bar@example.com")
        self.assertEqual(
            raisedexp.exception.message, "You must be an admin to create an invitation to an organization."
        )
        self.assertEqual(raisedexp.exception.status, 403)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/orgs/members/#create-organization-invitation",
                "message": "You must be an admin to create an invitation to an organization.",
            },
        )

    def testCreateMigration(self):
        self.org = self.g.get_organization("sample-test-organisation")
        self.assertTrue(isinstance(self.org.create_migration(["sample-repo"]), github.Migration.Migration))

    def testGetMigrations(self):
        self.org = self.g.get_organization("sample-test-organisation")
        self.assertEqual(self.org.get_migrations().totalCount, 2)

    def testGetInstallations(self):
        installations = self.org.get_installations()
        self.assertEqual(installations[0].id, 123456)
        self.assertEqual(installations[0].app_id, 10101)
        self.assertEqual(installations[0].target_id, 3344556)
        self.assertEqual(installations[0].target_type, "User")
        self.assertEqual(installations.totalCount, 1)

    def testCreateVariable(self):
        variable = self.org.create_variable("variable-name", "variable-value", "all")
        self.assertIsNotNone(variable)

    def testCreateVariableSelected(self):
        repos = [self.org.get_repo("TestPyGithub"), self.org.get_repo("FatherBeaver")]
        variable = self.org.create_variable("variable-name", "variable-value", "selected", repos)
        self.assertIsNotNone(variable)
        self.assertEqual(list(variable.selected_repositories), repos)

    def testGetVariable(self):
        repos = [self.org.get_repo("TestPyGithub"), self.org.get_repo("FatherBeaver")]
        variable = self.org.get_variable("variable-name")
        self.assertEqual(variable.name, "variable-name")
        self.assertEqual(variable.created_at, datetime(2019, 8, 10, 14, 59, 22, tzinfo=timezone.utc))
        self.assertEqual(variable.updated_at, datetime(2020, 1, 10, 14, 59, 22, tzinfo=timezone.utc))
        self.assertEqual(variable.visibility, "selected")
        self.assertEqual(list(variable.selected_repositories), repos)
        self.assertEqual(variable.url, "https://api.github.com/orgs/BeaverSoftware/actions/variables/variable-name")

    def testGetLazyVariable(self):
        var = self.g.withLazy(True).get_organization("org").get_variable("var name")
        self.assertEqual(str(var), 'OrganizationVariable(name="var name")')
        self.assertEqual(var.name, "var name")
        self.assertEqual(var.url, "/orgs/org/actions/variables/var%20name")

    def testGetVariables(self):
        variables = self.org.get_variables()
        self.assertEqual(len(list(variables)), 1)

    @mock.patch("github.PublicKey.encrypt")
    def testCreateActionsSecret(self, encrypt):
        org = self.g.get_organization("demoorg")
        # encrypt returns a non-deterministic value, we need to mock it so the replay data matches
        encrypt.return_value = "M+5Fm/BqTfB90h3nC7F3BoZuu3nXs+/KtpXwxm9gG211tbRo0F5UiN0OIfYT83CKcx9oKES9Va4E96/b"
        secret = org.create_secret("secret_name", "secret-value", visibility="all")
        self.assertIsNotNone(secret)

    @mock.patch("github.PublicKey.encrypt")
    def testCreateDependabotSecret(self, encrypt):
        org = self.g.get_organization("demoorg")
        # encrypt returns a non-deterministic value, we need to mock it so the replay data matches
        encrypt.return_value = "M+5Fm/BqTfB90h3nC7F3BoZuu3nXs+/KtpXwxm9gG211tbRo0F5UiN0OIfYT83CKcx9oKES9Va4E96/b"
        secret = org.create_secret("secret_name", "secret-value", secret_type="dependabot", visibility="all")
        self.assertIsNotNone(secret)

    def testOrgGetSecretAssertion(self):
        org = self.g.get_organization("demoorg")
        with self.assertRaises(AssertionError) as exc:
            org.get_secret(secret_name="splat", secret_type="supersecret")
        self.assertEqual(str(exc.exception), "secret_type should be actions or dependabot")

    @mock.patch("github.PublicKey.encrypt")
    def testCreateDependabotSecretSelected(self, encrypt):
        org = self.g.get_organization("demoorg")
        repos = [org.get_repo("demo-repo-1"), org.get_repo("demo-repo-2")]
        # encrypt returns a non-deterministic value, we need to mock it so the replay data matches
        encrypt.return_value = "M+5Fm/BqTfB90h3nC7F3BoZuu3nXs+/KtpXwxm9gG211tbRo0F5UiN0OIfYT83CKcx9oKES9Va4E96/b"
        secret = org.create_secret(
            secret_name="SECRET_DEP_NAME",
            unencrypted_value="secret-value",
            visibility="selected",
            secret_type="dependabot",
            selected_repositories=repos,
        )

        self.assertIsNotNone(secret)
        self.assertEqual(secret.visibility, "selected")
        self.assertEqual(list(secret.selected_repositories), repos)

    @mock.patch("github.PublicKey.encrypt")
    def testOrgSecretEdit(self, encrypt):
        org = self.g.get_organization("demoorg")
        repos = [org.get_repo("demo-repo-1"), org.get_repo("demo-repo-2")]
        # encrypt returns a non-deterministic value, we need to mock it so the replay data matches
        encrypt.return_value = "M+5Fm/BqTfB90h3nC7F3BoZuu3nXs+/KtpXwxm9gG211tbRo0F5UiN0OIfYT83CKcx9oKES9Va4E96/b"
        secret = org.create_secret(
            secret_name="secret_act_name",
            unencrypted_value="secret-value",
            visibility="selected",
            secret_type="actions",
            selected_repositories=repos,
        )

        with self.assertRaises(AssertionError) as exc:
            secret.edit(value="newvalue", secret_type="supersecret")
        self.assertEqual(str(exc.exception), "secret_type should be actions or dependabot")

    def testCreateCustomProperties(self):
        properties = [
            CustomProperty(
                property_name="property_1",
                value_type="string",
                required=False,
                description="description",
                values_editable_by="org_actors",
            ),
            CustomProperty(
                property_name="property_2",
                value_type="single_select",
                required=True,
                default_value="bar",
                description="Lorem ipsum",
                allowed_values=["foo", "bar"],
                values_editable_by="org_and_repo_actors",
            ),
            CustomProperty(
                property_name="property_3",
                value_type="multi_select",
                required=True,
                default_value="bar",
                description="Lorem ipsum",
                allowed_values=["foo", "bar"],
                values_editable_by="org_and_repo_actors",
            ),
            CustomProperty(
                property_name="property_4",
                value_type="true_false",
                required=False,
                description="description",
                values_editable_by="org_actors",
            ),
        ]
        properties = self.org.create_custom_properties(properties)
        properties_map = {p.property_name: p for p in properties}
        property_1 = properties_map["property_1"]
        self.assertEqual(property_1.value_type, "string")
        property_2 = properties_map["property_2"]
        self.assertEqual(property_2.description, "Lorem ipsum")
        property_3 = properties_map["property_3"]
        self.assertEqual(property_3.value_type, "multi_select")
        property_4 = properties_map["property_4"]
        self.assertEqual(property_4.value_type, "true_false")

    def testCreateCustomProperty(self):
        custom_property = CustomProperty(
            property_name="property_1",
            value_type="string",
            required=True,
            default_value="foo",
            description="description",
        )
        created_property = self.org.create_custom_property(custom_property)
        self.assertEqual(created_property.property_name, "property_1")
        self.assertEqual(created_property.value_type, "string")
        self.assertEqual(created_property.required, True)
        self.assertEqual(created_property.default_value, "foo")
        self.assertEqual(created_property.description, "description")
        self.assertIsNone(created_property.url)
        self.assertEqual(created_property.values_editable_by, "org_actors")

    def testGetCustomProperties(self):
        properties = self.org.get_custom_properties()
        properties_map = {p.property_name: p for p in properties}
        self.assertIn("property_1", properties_map)
        self.assertIn("property_2", properties_map)

    def testGetCustomProperty(self):
        custom_property = self.org.get_custom_property("property_1")
        self.assertEqual(custom_property.property_name, "property_1")
        self.assertEqual(custom_property.value_type, "string")
        self.assertEqual(custom_property.required, True)
        self.assertEqual(custom_property.default_value, "foo")
        self.assertEqual(custom_property.description, "description")
        self.assertEqual(custom_property.url, "https://api.github.com/orgs/BeaverSoftware/properties/schema/property_1")
        self.assertEqual(custom_property.values_editable_by, "org_actors")

    def testCreateCustomPropertyValues(self):
        self.org.create_custom_property_values(["TestPyGithub"], {"property_1": "bar"})
        self.testListCustomPropertyValues()

    def testListCustomPropertyValues(self):
        repos = list(self.org.list_custom_property_values("repo:BeaverSoftware/TestPyGithub"))
        repos_map = {r.repository_name: r for r in repos}
        self.assertIn("TestPyGithub", repos_map)
        self.assertIn("property_1", repos_map["TestPyGithub"].properties)
        self.assertEqual(repos_map["TestPyGithub"].properties["property_1"], "bar")

    def testRemoveCustomProperty(self):
        self.org.remove_custom_property("property_1")
        with self.assertRaises(github.UnknownObjectException):
            self.org.get_custom_property("property_1")

    def testGetSelfHostedRunners(self):
        runners = self.org.get_self_hosted_runners()
        self.assertEqual(runners.totalCount, 602)

    def testDeleteSelfHostedRunner(self):
        self.org.delete_self_hosted_runner("42")

    def testGetSelfHostedRunnerApplications(self):
        self.assertListKeyEqual(
            self.org.get_self_hosted_runner_applications(),
            lambda h: h.os,
            ["osx", "linux", "linux", "win", "linux", "osx", "win"],
        )

    def testSelfHostedRunnerJitConfig(self):
        runner = self.org.create_self_hosted_runner_jitconfig(name="self_hosted", runner_group_id=1, labels=["default"])
        # Now remove the runner
        for runner in self.org.get_self_hosted_runners():
            if runner.name == "self_hosted":
                runner = self.org.get_self_hosted_runner(runner_id=runner.id)
                self.org.delete_self_hosted_runner(runner_id=runner.id)

    def testSelfHostedRunnerGetRegistrationToken(self):
        token = self.org.create_self_hosted_runner_registration_token()
        self.assertEqual(token.token, "XXXXXX")
        self.assertEqual(
            token.expires_at, datetime(2025, 2, 17, 21, 11, 49, 260000, tzinfo=timezone(timedelta(hours=-8)))
        )

    def testSelfHostedRunnerGetRemoveToken(self):
        token = self.org.create_self_hosted_runner_remove_token()
        self.assertEqual(token.token, "XXXXXX")
        self.assertEqual(
            token.expires_at, datetime(2025, 2, 17, 21, 12, 28, 308000, tzinfo=timezone(timedelta(hours=-8)))
        )

    def testGetCodeSecurityConfigs(self):
        configs = list(self.org.get_code_security_configs())
        self.assertEqual(configs.pop().id, 17)

    def testCreateCodeSecurityConfigs(self):
        config = self.org.create_code_security_config(name="test1", description="This is a description")
        self.assertEqual(config.name, "test1")

        self.org.delete_code_security_config(id=config.id)

    def testGetCodeSecurityConfig(self):
        config = self.org.get_code_security_config(id=17)
        self.assertEqual(config.id, 17)

    def testSetDefaultCodeSecurityConfig(self):
        self.org.set_default_code_security_config(id=17, default_for_new_repos="all")
        configs = self.org.get_default_code_security_configs()
        for config in configs:
            if config.default_for_new_repos == "all":
                self.assertEqual(config.configuration.id, 17)

    def testAttachDetachSecurityConfig(self):
        config = self.org.create_code_security_config(name="test1", description="This is a description")
        repo = self.org.get_repo("test1")
        repo.attach_security_config(id=config.id)
        status = "unknown"
        while status != "enforced":
            repo_config = repo.get_security_config()
            if repo_config:
                status = repo_config.status
            else:
                status = "unknown"

        self.assertEqual(config.id, repo_config.configuration.id)
        repo.detach_security_config()

    def testGetReposForCodeSecurityConfig(self):
        repo_statuses = self.org.get_repos_for_code_security_config(id=182032)
        status = repo_statuses[0]
        self.assertEqual(status.status, "enforced")
        self.assertIsNotNone(status.repository)
        self.assertEqual(status.repository.full_name, "BeaverSoftware/truth")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class PullRequestFile(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.file = self.g.get_user().get_repo("PyGithub").get_pull(31).get_files()[0]

    def testAttributes(self):
        self.assertEqual(self.file.additions, 1)
        self.assertEqual(
            self.file.blob_url,
            "https://github.com/jacquev6/PyGithub/blob/8a4f306d4b223682dd19410d4a9150636ebe4206/codegen/templates/GithubObject.py",
        )
        self.assertEqual(self.file.changes, 2)
        self.assertEqual(self.file.deletions, 1)
        self.assertEqual(self.file.filename, "codegen/templates/GithubObject.py")
        self.assertEqual(
            self.file.patch,
            '@@ -70,7 +70,7 @@ def __useAttributes( self, attributes ):\n \n         # @toto No need to check if attribute is in attributes when attribute is mandatory\n {% for attribute in class.attributes|dictsort:"name" %}\n-        if "{{ attribute.name }}" in attributes and attributes[ "{{ attribute.name }}" ] is not None:\n+        if "{{ attribute.name }}" in attributes and attributes[ "{{ attribute.name }}" ] is not None: # pragma no branch\n \n {% if attribute.type.cardinality == "scalar" %}\n {% if attribute.type.simple %}',
        )
        self.assertEqual(
            self.file.raw_url,
            "https://github.com/jacquev6/PyGithub/raw/8a4f306d4b223682dd19410d4a9150636ebe4206/codegen/templates/GithubObject.py",
        )
        self.assertEqual(self.file.sha, "8a4f306d4b223682dd19410d4a9150636ebe4206")
        self.assertEqual(self.file.status, "modified")
        self.assertEqual(
            repr(self.file),
            'File(sha="8a4f306d4b223682dd19410d4a9150636ebe4206", filename="codegen/templates/GithubObject.py")',
        )



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jeppe Fihl-Pearson <tenzer@tenzer.dk>                         #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Xavi Vega <xabi1309@gmail.com>                                #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from datetime import datetime, timezone

from . import Framework


class WorkflowJob(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub")
        self.job = self.repo.get_workflow_run(4205440316).jobs()[0]

    def testAttributes(self):
        self.assertEqual(self.job.id, 11421878319)
        self.assertEqual(self.job.run_id, 4205440316)
        self.assertEqual(
            self.job.run_url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/runs/4205440316",
        )
        self.assertEqual(self.job.node_id, "CR_kwDOGpsAJ88AAAACqMwILw")
        self.assertEqual(self.job.head_sha, "06ec040b2eeef6c0316dd5abcda0608525a3f205")
        self.assertEqual(
            self.job.url,
            "https://api.github.com/repos/PyGithub/PyGithub/actions/jobs/11421878319",
        )
        self.assertEqual(
            self.job.html_url,
            "https://github.com/PyGithub/PyGithub/actions/runs/4205440316/jobs/7297536068",
        )
        self.assertEqual(self.job.status, "completed")
        self.assertEqual(self.job.conclusion, "success")
        started_at = datetime(2023, 2, 17, 16, 3, 46, tzinfo=timezone.utc)
        self.assertEqual(self.job.started_at, started_at)
        completed_at = datetime(2023, 2, 17, 16, 4, 52, tzinfo=timezone.utc)
        self.assertEqual(self.job.completed_at, completed_at)
        self.assertEqual(self.job.name, "test (Python 3.7)")
        self.assertEqual(
            self.job.check_run_url,
            "https://api.github.com/repos/PyGithub/PyGithub/check-runs/11421878319",
        )
        self.assertListKeyEqual(
            self.job.steps,
            lambda s: s.name,
            [
                "Set up job",
                "Run actions/checkout@v2",
                "Set up Python",
                "Install tox",
                "Run tests",
                "Upload coverage to Codecov",
                "Post Set up Python",
                "Post Run actions/checkout@v2",
                "Complete job",
            ],
        )
        self.assertEqual(
            self.job.logs_url(),
            "https://pipelines.actions.githubusercontent.com/serviceHosts/d560a817-28d4-4544-a539-eb35c2a56899/_apis/pipelines/1/runs/5/signedlogcontent/5?urlExpires=2023-03-15T17%3A02%3A58.1305046Z&urlSigningMethod=HMACV1&urlSignature=abcdefghijklmn",
        )
        self.assertEqual(self.job.runner_id, 2)
        self.assertEqual(self.job.runner_name, "GitHub Actions 2")
        self.assertEqual(self.job.runner_group_id, 2)
        self.assertEqual(self.job.runner_group_name, "GitHub Actions")
        created_at = datetime(2023, 2, 17, 16, 3, 38, tzinfo=timezone.utc)
        self.assertEqual(self.job.created_at, created_at)
        self.assertEqual(self.job.head_branch, "tz-aware-2")
        self.assertEqual(self.job.labels, ["ubuntu-latest"])
        self.assertEqual(self.job.run_attempt, 1)
        self.assertEqual(self.job.workflow_name, "CI")



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Stuart Glaser <stuglaser@gmail.com>                           #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 @tmshn <tmshn@r.recruit.co.jp>                                #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Matt Babineau <babineaum@users.noreply.github.com>            #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Nicolas Agustín Torres <nicolastrres@gmail.com>               #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Filipe Laíns <filipe.lains@gmail.com>                         #
# Copyright 2019 Nick Campbell <nicholas.j.campbell@gmail.com>                 #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Huan-Cheng Chang <changhc84@gmail.com>                        #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2023 Nicolas Schweitzer <nicolas.schweitzer@datadoghq.com>         #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2024 Malik Shahzad Muzaffar <shahzad.malik.muzaffar@cern.ch>       #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from datetime import datetime, timezone

from . import Framework

gho = Framework.github.GithubObject


class Issue(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_repo("PyGithub/PyGithub")
        self.issue = self.repo.get_issue(28)

    def testAttributes(self):
        self.assertIsNone(self.issue.active_lock_reason)
        self.assertEqual(self.issue.assignee.login, "jacquev6")
        self.assertListKeyEqual(self.issue.assignees, lambda a: a.login, ["jacquev6"])
        self.assertEqual(self.issue.author_association, "MEMBER")
        self.assertEqual(self.issue.body, "Body edited by PyGithub\n")
        self.assertIsNone(self.issue.body_html)
        self.assertIsNone(self.issue.body_text)
        self.assertEqual(self.issue.closed_at, datetime(2012, 5, 26, 14, 59, 33, tzinfo=timezone.utc))
        self.assertEqual(self.issue.closed_by.login, "jacquev6")
        self.assertEqual(self.issue.comments, 3)
        self.assertEqual(self.issue.comments_url, "https://api.github.com/repos/PyGithub/PyGithub/issues/28/comments")
        self.assertEqual(self.issue.created_at, datetime(2012, 5, 19, 10, 38, 23, tzinfo=timezone.utc))
        self.assertIsNone(self.issue.draft)
        self.assertEqual(self.issue.events_url, "https://api.github.com/repos/PyGithub/PyGithub/issues/28/events")
        self.assertEqual(self.issue.html_url, "https://github.com/PyGithub/PyGithub/issues/28")
        self.assertEqual(self.issue.id, 4653757)
        self.assertEqual(self.issue.issue_dependencies_summary.blocking, 0)
        self.assertEqual(self.issue.issue_dependencies_summary.blocked_by, 0)
        self.assertEqual(self.issue.issue_dependencies_summary.total_blocking, 1)
        self.assertEqual(self.issue.issue_dependencies_summary.total_blocked_by, 1)
        self.assertListKeyEqual(self.issue.labels, lambda lb: lb.name, ["bug", "question"])
        self.assertEqual(
            self.issue.labels_url, "https://api.github.com/repos/PyGithub/PyGithub/issues/28/labels{/name}"
        )
        self.assertEqual(self.issue.locked, False)
        self.assertEqual(self.issue.milestone.title, "Version 1.4")
        self.assertEqual(self.issue.node_id, "MDU6SXNzdWU0NjUzNzU3")
        self.assertEqual(self.issue.number, 28)
        self.assertIsNone(self.issue.parent_issue_url)
        self.assertIsNone(self.issue.performed_via_github_app)
        self.assertIsNone(self.issue.pull_request)
        self.assertEqual(
            self.issue.reactions,
            {
                "url": "https://api.github.com/repos/PyGithub/PyGithub/issues/28/reactions",
                "total_count": 2,
                "+1": 0,
                "-1": 0,
                "laugh": 0,
                "hooray": 2,
                "confused": 0,
                "heart": 0,
                "rocket": 0,
                "eyes": 0,
            },
        )
        self.assertEqual(self.issue.repository.full_name, "PyGithub/PyGithub")
        self.assertEqual(self.issue.repository_url, "https://api.github.com/repos/PyGithub/PyGithub")
        self.assertEqual(self.issue.state, "closed")
        self.assertEqual(self.issue.state_reason, "completed")
        self.assertEqual(self.issue.sub_issues_summary.completed, 1)
        self.assertEqual(self.issue.sub_issues_summary.percent_completed, 100)
        self.assertEqual(self.issue.sub_issues_summary.total, 1)
        self.assertIsNone(self.issue.text_matches)
        self.assertEqual(self.issue.timeline_url, "https://api.github.com/repos/PyGithub/PyGithub/issues/28/timeline")
        self.assertEqual(self.issue.title, "Issue created by PyGithub")
        self.assertEqual(self.issue.type.name, "Bug")
        self.assertEqual(self.issue.updated_at, datetime(2025, 8, 15, 19, 35, 40, tzinfo=timezone.utc))
        self.assertEqual(self.issue.url, "https://api.github.com/repos/PyGithub/PyGithub/issues/28")
        self.assertFalse(self.issue.locked)
        self.assertIsNone(self.issue.active_lock_reason)
        self.assertEqual(self.issue.user.login, "jacquev6")
        self.assertEqual(self.issue.repository.name, "PyGithub")
        self.assertEqual(repr(self.issue), 'Issue(title="Issue created by PyGithub", number=28)')
        self.assertEqual(
            self.issue.reactions,
            {
                "url": "https://api.github.com/repos/PyGithub/PyGithub/issues/28/reactions",
                "total_count": 2,
                "+1": 0,
                "-1": 0,
                "laugh": 0,
                "hooray": 2,
                "confused": 0,
                "heart": 0,
                "rocket": 0,
                "eyes": 0,
            },
        )

    def testLazyAttributes(self):
        issue = self.g.withLazy(True).get_repo("lazy/repo").get_issue(42)
        self.assertEqual(str(issue), "Issue(title=None, number=42)")
        self.assertEqual(issue._identity, 42)
        self.assertEqual(issue.number, 42)
        self.assertEqual(issue.url, "/repos/lazy/repo/issues/42")

    def testEditWithoutParameters(self):
        self.issue.edit()

    def testEditWithAllParameters(self):
        user = self.g.get_user("jacquev6")
        milestone = self.repo.get_milestone(2)
        self.issue.edit(
            "Title edited by PyGithub",
            "Body edited by PyGithub",
            gho.NotSet,
            "closed",
            milestone,
            ["Bug"],
            [user],
            "completed",
        )
        self.assertListKeyEqual(self.issue.assignees, lambda a: a.login, ["jacquev6"])
        self.assertEqual(self.issue.body, "Body edited by PyGithub")
        self.assertEqual(self.issue.state, "closed")
        self.assertEqual(self.issue.state_reason, "completed")
        self.assertEqual(self.issue.title, "Title edited by PyGithub")
        self.assertListKeyEqual(self.issue.labels, lambda lb: lb.name, ["Bug"])

    def testEditResetMilestone(self):
        self.assertEqual(self.issue.milestone.title, "Version 1.4")
        self.issue.edit(milestone=None)
        self.assertEqual(self.issue.milestone, None)

    def testEditResetAssignees(self):
        self.assertEqual(len(self.issue.assignees), 1)
        self.issue.edit(assignees=[])
        self.assertEqual(len(self.issue.assignees), 0)

    def testEditWithStateReasonNotPlanned(self):
        self.issue.edit(state="closed", state_reason="not_planned")
        self.assertEqual(self.issue.state, "closed")
        self.assertEqual(self.issue.state_reason, "not_planned")

    def testEditWithStateReasonReopened(self):
        self.issue.edit(state="open", state_reason="reopened")
        self.assertEqual(self.issue.state, "open")
        self.assertEqual(self.issue.state_reason, "reopened")

    def testLock(self):
        self.issue.lock("resolved")

    def testUnlock(self):
        self.issue.unlock()

    def testCreateComment(self):
        comment = self.issue.create_comment("Comment created by PyGithub")
        self.assertEqual(comment.id, 5808311)

    def testGetComments(self):
        self.assertListKeyEqual(self.issue.get_comments(), lambda c: c.user.login, ["jacquev6", "roskakori"])

    def testGetCommentsSince(self):
        self.assertListKeyEqual(
            self.issue.get_comments(datetime(2012, 5, 26, 13, 59, 33, tzinfo=timezone.utc)),
            lambda c: c.user.login,
            ["jacquev6", "roskakori"],
        )

    def testGetEvents(self):
        self.assertListKeyEqual(self.issue.get_events(), lambda e: e.id, [15819975, 15820048])

    def testGetLabels(self):
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )

    def testAddAndRemoveAssignees(self):
        user1 = "jayfk"
        user2 = self.g.get_user("jzelinskie")
        self.assertListKeyEqual(self.issue.assignees, lambda a: a.login, ["jacquev6"])
        self.issue.add_to_assignees(user1, user2)
        self.assertListKeyEqual(
            self.issue.assignees,
            lambda a: a.login,
            ["jacquev6", "jayfk", "jzelinskie"],
        )
        self.issue.remove_from_assignees(user1, user2)
        self.assertListKeyEqual(self.issue.assignees, lambda a: a.login, ["jacquev6"])

    def testAddAndRemoveLabels(self):
        bug = self.repo.get_label("Bug")
        question = self.repo.get_label("Question")
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )
        self.issue.remove_from_labels(bug)
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Project management", "Question"],
        )
        self.issue.remove_from_labels(question)
        self.assertListKeyEqual(self.issue.get_labels(), lambda lb: lb.name, ["Project management"])
        self.issue.add_to_labels(bug, question)
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )

    def testAddAndRemoveLabelsWithStringArguments(self):
        bug = "Bug"
        question = "Question"
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )
        self.issue.remove_from_labels(bug)
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Project management", "Question"],
        )
        self.issue.remove_from_labels(question)
        self.assertListKeyEqual(self.issue.get_labels(), lambda lb: lb.name, ["Project management"])
        self.issue.add_to_labels(bug, question)
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )

    def testDeleteAndSetLabels(self):
        bug = self.repo.get_label("Bug")
        question = self.repo.get_label("Question")
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )
        self.issue.delete_labels()
        self.assertListKeyEqual(self.issue.get_labels(), None, [])
        self.issue.set_labels(bug, question)
        self.assertListKeyEqual(self.issue.get_labels(), lambda lb: lb.name, ["Bug", "Question"])

    def testDeleteAndSetLabelsWithStringArguments(self):
        bug = "Bug"
        question = "Question"
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", "Project management", "Question"],
        )
        self.issue.delete_labels()
        self.assertListKeyEqual(self.issue.get_labels(), None, [])
        self.issue.set_labels(bug, question)
        self.assertListKeyEqual(self.issue.get_labels(), lambda lb: lb.name, ["Bug", "Question"])

    def testGetReactions(self):
        reactions = self.issue.get_reactions()
        self.assertEqual(reactions[0].content, "+1")

    def testCreateReaction(self):
        reaction = self.issue.create_reaction("hooray")

        self.assertEqual(reaction.id, 16917472)
        self.assertEqual(reaction.content, "hooray")

    def testDeleteReaction(self):
        self.assertTrue(self.issue.delete_reaction(85740167))

    def testGetTimeline(self):
        expected_events = {
            "referenced",
            "cross-referenced",
            "locked",
            "unlocked",
            "closed",
            "assigned",
            "commented",
            "subscribed",
            "labeled",
        }
        events = self.issue.get_timeline()

        first = events[0]
        self.assertEqual(15819975, first.id)
        self.assertEqual("MDE1OlN1YnNjcmliZWRFdmVudDE1ODE5OTc1", first.node_id)
        self.assertEqual(
            "https://api.github.com/repos/PyGithub/PyGithub/issues/events/15819975",
            first.url,
        )
        self.assertEqual("jacquev6", first.actor.login)
        self.assertEqual(327146, first.actor.id)
        self.assertEqual("subscribed", first.event)
        self.assertIsNone(first.commit_id)
        self.assertIsNone(first.commit_url)
        self.assertEqual(repr(first), "TimelineEvent(id=15819975)")

        for event in events:
            self.assertIn(event.event, expected_events)
            self.assertIsNotNone(event.created_at)
            self.assertIsNotNone(event.actor)

            if event.event == "cross-referenced":
                # cross-referenced events don't include an event id or node_id
                self.assertIsNotNone(event.source)
                self.assertEqual(event.source.type, "issue")
                self.assertEqual(event.source.issue.number, 857)
                self.assertEqual(repr(event.source), 'TimelineEventSource(type="issue")')
            else:
                self.assertIsNotNone(event.id)
                self.assertIsNotNone(event.node_id)

                if event.event == "commented":
                    self.assertIsNotNone(event.body)
                else:
                    self.assertIsNone(event.source)
                    self.assertIsNotNone(event.actor)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Benjamin K <53038537+treee111@users.noreply.github.com>       #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

from . import Framework


class BranchProtection(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.branch_protection = self.g.get_repo("curvewise-forks/PyGithub").get_branch("master").get_protection()

    def testAttributes(self):
        self.assertEqual(self.branch_protection.allow_deletions, False)
        self.assertEqual(self.branch_protection.allow_force_pushes, False)
        self.assertEqual(self.branch_protection.allow_fork_syncing, False)
        self.assertEqual(self.branch_protection.block_creations, False)
        self.assertIsNone(self.branch_protection.enabled)
        self.assertEqual(self.branch_protection.enforce_admins, True)
        self.assertEqual(self.branch_protection.lock_branch, False)
        self.assertIsNone(self.branch_protection.name)
        self.assertIsNone(self.branch_protection.protection_url)
        self.assertEqual(self.branch_protection.required_conversation_resolution, False)
        self.assertEqual(self.branch_protection.required_linear_history, True)
        self.assertEqual(
            self.branch_protection.required_pull_request_reviews.url,
            "https://api.github.com/repos/curvewise-forks/PyGithub/branches/master/protection/required_pull_request_reviews",
        )
        self.assertEqual(self.branch_protection.required_signatures, False)
        self.assertTrue(self.branch_protection.required_status_checks.strict)
        self.assertEqual(self.branch_protection.required_status_checks.contexts, ["build (3.10)"])
        self.assertTrue(self.branch_protection.required_linear_history)
        self.assertIsNone(self.branch_protection.restrictions)
        self.assertEqual(
            self.branch_protection.url,
            "https://api.github.com/repos/curvewise-forks/PyGithub/branches/master/protection",
        )
        self.assertEqual(
            self.branch_protection.__repr__(),
            'BranchProtection(url="https://api.github.com/repos/curvewise-forks/PyGithub/branches/master/protection")',
        )
        self.assertFalse(self.branch_protection.allow_force_pushes)
        self.assertFalse(self.branch_protection.allow_deletions)
        self.assertFalse(self.branch_protection.required_conversation_resolution)
        self.assertFalse(self.branch_protection.lock_branch)
        self.assertFalse(self.branch_protection.allow_fork_syncing)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class Issue50(Framework.TestCase):  # https://github.com/jacquev6/PyGithub/issues/50
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user().get_repo("PyGithub")
        self.issue = self.repo.get_issue(50)
        self.labelName = "Label with spaces and strange characters (&*#$)"

    def testCreateLabel(self):
        label = self.repo.create_label(self.labelName, "ffff00")
        self.assertEqual(label.name, self.labelName)

    def testGetLabel(self):
        label = self.repo.get_label(self.labelName)
        self.assertEqual(label.name, self.labelName)

    def testGetLabels(self):
        self.assertListKeyEqual(
            self.repo.get_labels(),
            lambda lb: lb.name,
            [
                "Refactoring",
                "Public interface",
                "Functionalities",
                "Project management",
                "Bug",
                "Question",
                "RequestedByUser",
                self.labelName,
            ],
        )

    def testAddLabelToIssue(self):
        self.issue.add_to_labels(self.repo.get_label(self.labelName))

    def testRemoveLabelFromIssue(self):
        self.issue.remove_from_labels(self.repo.get_label(self.labelName))

    def testSetIssueLabels(self):
        self.issue.set_labels(
            self.repo.get_label("Bug"),
            self.repo.get_label("RequestedByUser"),
            self.repo.get_label(self.labelName),
        )

    def testIssueLabels(self):
        self.assertListKeyEqual(
            self.issue.labels,
            lambda lb: lb.name,
            ["Bug", self.labelName, "RequestedByUser"],
        )

    def testIssueGetLabels(self):
        self.assertListKeyEqual(
            self.issue.get_labels(),
            lambda lb: lb.name,
            ["Bug", self.labelName, "RequestedByUser"],
        )

    def testGetIssuesWithLabel(self):
        self.assertListKeyEqual(
            self.repo.get_issues(labels=[self.repo.get_label(self.labelName)]),
            lambda i: i.number,
            [52, 50],
        )

    def testCreateIssueWithLabel(self):
        issue = self.repo.create_issue(
            "Issue created by PyGithub to test issue #50",
            labels=[self.repo.get_label(self.labelName)],
        )
        self.assertListKeyEqual(issue.labels, lambda lb: lb.name, [self.labelName])
        self.assertEqual(issue.number, 52)



############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2015 Kyle Hornberg <khornberg@users.noreply.github.com>            #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Alice GIRARD <bouhahah@gmail.com>                             #
# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2019 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2019 TechnicalPirate <35609336+TechnicalPirate@users.noreply.github.com>#
# Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
# Copyright 2024 Benjamin K <53038537+treee111@users.noreply.github.com>       #
# Copyright 2024 Benjamin K. <53038537+treee111@users.noreply.github.com>      #
# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
# Copyright 2025 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from __future__ import annotations

import github

from . import Framework


class Branch(Framework.TestCase):
    def setUp(self):
        super().setUp()
        self.repo = self.g.get_user().get_repo("PyGithub")
        self.branch = self.repo.get_branch("topic/RewriteWithGeneratedCode")
        self.protected_branch = self.repo.get_branch("integrations")
        self.organization_branch = self.g.get_repo("PyGithub/PyGithub", lazy=True).get_branch("master")

    def testAttributes(self):
        self.assertEqual(
            self.branch._links,
            {
                "self": "https://api.github.com/repos/jacquev6/PyGithub/branches/topic/RewriteWithGeneratedCode",
                "html": "https://github.com/jacquev6/PyGithub/tree/topic/RewriteWithGeneratedCode",
            },
        )
        self.assertEqual(self.branch.commit.sha, "f23da453917a36c8bd48ab8d99e5fa7221884342")
        self.assertEqual(self.branch.name, "topic/RewriteWithGeneratedCode")
        self.assertEqual(self.branch.commit.sha, "f23da453917a36c8bd48ab8d99e5fa7221884342")
        self.assertIsNone(self.branch.pattern)
        self.assertEqual(self.branch.protected, False)
        self.assertIsNone(self.branch.protection.url)
        self.assertEqual(
            self.branch.protection_url,
            "https://api.github.com/repos/jacquev6/PyGithub/branches/topic/RewriteWithGeneratedCode/protection",
        )
        self.assertFalse(self.branch.protected)
        self.assertEqual(repr(self.branch), 'Branch(name="topic/RewriteWithGeneratedCode")')
        self.assertIsNone(self.branch.required_approving_review_count)

    def testEditProtection(self):
        self.protected_branch.edit_protection(
            strict=True,
            require_code_owner_reviews=True,
            required_approving_review_count=2,
            require_last_push_approval=True,
        )
        branch_protection = self.protected_branch.get_protection()
        self.assertTrue(branch_protection.required_status_checks.strict)
        self.assertEqual(branch_protection.required_status_checks.checks, [])
        self.assertEqual(branch_protection.required_status_checks.contexts, [])
        self.assertEqual(
            branch_protection.required_status_checks.contexts_url,
            "https://api.github.com/repos/jacquev6/PyGithub/branches/integrations/protection/required_status_checks/contexts",
        )
        self.assertTrue(branch_protection.enforce_admins)
        self.assertFalse(branch_protection.required_linear_history)
        self.assertFalse(branch_protection.allow_deletions)
        self.assertFalse(branch_protection.required_pull_request_reviews.dismiss_stale_reviews)
        self.assertTrue(branch_protection.required_pull_request_reviews.require_code_owner_reviews)
        self.assertEqual(
            branch_protection.required_pull_request_reviews.required_approving_review_count,
            2,
        )
        self.assertTrue(branch_protection.required_pull_request_reviews.require_last_push_approval)
        self.assertEqual(
            branch_protection.required_pull_request_reviews.url,
            "https://api.github.com/repos/jacquev6/PyGithub/branches/integrations/protection/required_pull_request_reviews",
        )

    def testEditProtectionDismissalUsersWithUserOwnedBranch(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.protected_branch.edit_protection(dismissal_users=["jacquev6"])
        self.assertEqual(raisedexp.exception.message, "Validation Failed")
        self.assertEqual(raisedexp.exception.status, 422)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#update-branch-protection",
                "message": "Validation Failed",
                "errors": ["Only organization repositories can have users and team restrictions"],
            },
        )

    def testEditProtectionPushRestrictionsWithUserOwnedBranch(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.protected_branch.edit_protection(user_push_restrictions=["jacquev6"], team_push_restrictions=[])
        self.assertEqual(raisedexp.exception.message, "Validation Failed")
        self.assertEqual(raisedexp.exception.status, 422)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#update-branch-protection",
                "message": "Validation Failed",
                "errors": ["Only organization repositories can have users and team restrictions"],
            },
        )

    def testEditProtectionPushRestrictionsAndDismissalUser(self):
        self.organization_branch.edit_protection(dismissal_users=["jacquev6"], user_push_restrictions=["jacquev6"])
        branch_protection = self.organization_branch.get_protection()
        self.assertListKeyEqual(
            branch_protection.required_pull_request_reviews.dismissal_users,
            lambda u: u.login,
            ["jacquev6"],
        )
        self.assertListKeyEqual(
            branch_protection.required_pull_request_reviews.dismissal_teams,
            lambda u: u.slug,
            [],
        )
        self.assertListKeyEqual(
            branch_protection.get_user_push_restrictions(),
            lambda u: u.login,
            ["jacquev6"],
        )
        self.assertListKeyEqual(branch_protection.get_team_push_restrictions(), lambda u: u.slug, [])

    def testRemoveProtection(self):
        self.assertTrue(self.protected_branch.protected)
        self.protected_branch.remove_protection()
        protected_branch = self.repo.get_branch("integrations")
        self.assertFalse(protected_branch.protected)
        with self.assertRaises(github.GithubException) as raisedexp:
            protected_branch.get_protection()
        self.assertEqual(raisedexp.exception.message, "Branch not protected")
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#get-branch-protection",
                "message": "Branch not protected",
            },
        )

    def testEditRequiredStatusChecks(self):
        self.protected_branch.edit_required_status_checks(strict=True)
        required_status_checks = self.protected_branch.get_required_status_checks()
        self.assertTrue(required_status_checks.strict)
        self.assertEqual(required_status_checks.contexts, ["foo/bar"])

    def testEditRequiredStatusChecksContexts(self):
        self.protected_branch.edit_required_status_checks(contexts=["check1", "check2"])
        required_status_checks = self.protected_branch.get_required_status_checks()
        self.assertEqual(required_status_checks.contexts, ["check1", "check2"])

    def testEditRequiredStatusChecksChecks(self):
        self.protected_branch.edit_required_status_checks(checks=["check1", ("check2", -1), ("check3", 123456)])
        required_status_checks = self.protected_branch.get_required_status_checks()
        self.assertEqual(required_status_checks.contexts, ["check1", "check2", "check3"])

    def testRemoveRequiredStatusChecks(self):
        self.protected_branch.remove_required_status_checks()
        with self.assertRaises(github.GithubException) as raisedexp:
            self.protected_branch.get_required_status_checks()
        self.assertEqual(raisedexp.exception.message, "Required status checks not enabled")
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#get-required-status-checks-of-protected-branch",
                "message": "Required status checks not enabled",
            },
        )

    def testEditRequiredPullRequestReviews(self):
        self.protected_branch.edit_required_pull_request_reviews(
            dismiss_stale_reviews=True,
            required_approving_review_count=2,
        )
        required_pull_request_reviews = self.protected_branch.get_required_pull_request_reviews()
        self.assertTrue(required_pull_request_reviews.dismiss_stale_reviews)
        self.assertTrue(required_pull_request_reviews.require_code_owner_reviews)
        self.assertEqual(required_pull_request_reviews.required_approving_review_count, 2)

    def testEditRequiredPullRequestReviewsWithTooLargeApprovingReviewCount(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.protected_branch.edit_required_pull_request_reviews(required_approving_review_count=9)
        self.assertEqual(raisedexp.exception.message, "Invalid request.\n\n9 must be less than or equal to 6.")
        self.assertEqual(raisedexp.exception.status, 422)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#update-pull-request-review-enforcement-of-protected-branch",
                "message": "Invalid request.\n\n9 must be less than or equal to 6.",
            },
        )

    def testEditRequiredPullRequestReviewsWithUserBranchAndDismissalUsers(self):
        with self.assertRaises(github.GithubException) as raisedexp:
            self.protected_branch.edit_required_pull_request_reviews(dismissal_users=["jacquev6"])
        self.assertEqual(
            raisedexp.exception.message,
            "Dismissal restrictions are supported only for repositories owned by an organization.",
        )
        self.assertEqual(raisedexp.exception.status, 422)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#update-pull-request-review-enforcement-of-protected-branch",
                "message": "Dismissal restrictions are supported only for repositories owned by an organization.",
            },
        )

    def testRemoveRequiredPullRequestReviews(self):
        self.protected_branch.remove_required_pull_request_reviews()
        required_pull_request_reviews = self.protected_branch.get_required_pull_request_reviews()
        self.assertFalse(required_pull_request_reviews.dismiss_stale_reviews)
        self.assertFalse(required_pull_request_reviews.require_code_owner_reviews)
        self.assertEqual(required_pull_request_reviews.required_approving_review_count, 1)
        self.assertFalse(required_pull_request_reviews.require_last_push_approval)

    def testAdminEnforcement(self):
        self.protected_branch.remove_admin_enforcement()
        self.assertFalse(self.protected_branch.get_admin_enforcement())
        self.protected_branch.set_admin_enforcement()
        self.assertTrue(self.protected_branch.get_admin_enforcement())

    def testAllowDeletions(self):
        self.protected_branch.set_allow_deletions()
        self.assertTrue(self.protected_branch.get_allow_deletions())
        self.protected_branch.remove_allow_deletions()
        self.assertFalse(self.protected_branch.get_allow_deletions())

    def testAddUserPushRestrictions(self):
        self.organization_branch.add_user_push_restrictions("sfdye")
        self.assertListKeyEqual(
            self.organization_branch.get_user_push_restrictions(),
            lambda u: u.login,
            ["jacquev6", "sfdye"],
        )

    def testReplaceUserPushRestrictions(self):
        self.assertListKeyEqual(
            self.organization_branch.get_user_push_restrictions(),
            lambda u: u.login,
            ["jacquev6"],
        )
        self.organization_branch.replace_user_push_restrictions("sfdye")
        self.assertListKeyEqual(
            self.organization_branch.get_user_push_restrictions(),
            lambda u: u.login,
            ["sfdye"],
        )

    def testRemoveUserPushRestrictions(self):
        self.organization_branch.remove_user_push_restrictions("jacquev6")
        self.assertListKeyEqual(
            self.organization_branch.get_user_push_restrictions(),
            lambda u: u.login,
            ["sfdye"],
        )

    def testAddTeamPushRestrictions(self):
        self.organization_branch.add_team_push_restrictions("pygithub-owners")
        self.assertListKeyEqual(
            self.organization_branch.get_team_push_restrictions(),
            lambda t: t.slug,
            ["pygithub-owners"],
        )

    def testReplaceTeamPushRestrictions(self):
        self.assertListKeyEqual(
            self.organization_branch.get_team_push_restrictions(),
            lambda t: t.slug,
            ["pygithub-owners"],
        )
        self.organization_branch.replace_team_push_restrictions("org-team")
        self.assertListKeyEqual(
            self.organization_branch.get_team_push_restrictions(),
            lambda t: t.slug,
            ["org-team"],
        )

    def testRemoveTeamPushRestrictions(self):
        self.organization_branch.remove_team_push_restrictions("org-team")
        self.assertListKeyEqual(
            self.organization_branch.get_team_push_restrictions(),
            lambda t: t.slug,
            ["pygithub-owners"],
        )

    def testRemovePushRestrictions(self):
        self.organization_branch.remove_push_restrictions()
        with self.assertRaises(github.GithubException) as raisedexp:
            list(self.organization_branch.get_user_push_restrictions())
        self.assertEqual(raisedexp.exception.message, "Push restrictions not enabled")
        self.assertEqual(raisedexp.exception.status, 404)
        self.assertEqual(
            raisedexp.exception.data,
            {
                "documentation_url": "https://developer.github.com/v3/repos/branches/#list-team-restrictions-of-protected-branch",
                "message": "Push restrictions not enabled",
            },
        )

    def testGetRequiredSignatures(self):
        required_signature = self.protected_branch.get_required_signatures()
        assert required_signature

    def testRemoveRequiredSignatures(self):
        self.protected_branch.remove_required_signatures()

    def testAddRequiredSignatures(self):
        self.protected_branch.add_required_signatures()
